# This file was automatically generated by SWIG (http://www.swig.org).
# Version 1.3.31
#
# Don't modify this file, modify the SWIG interface instead.
# This file is compatible with both classic and new-style classes.

import _TCPModbusClient
import new
new_instancemethod = new.instancemethod
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'PySwigObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static) or hasattr(self,name):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError,name

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

import types
try:
    _object = types.ObjectType
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0
del types


RCVBUFSIZE = _TCPModbusClient.RCVBUFSIZE
ARGS_QUERY = _TCPModbusClient.ARGS_QUERY
ARGS_READ = _TCPModbusClient.ARGS_READ
ARGS_WRITE = _TCPModbusClient.ARGS_WRITE
ARGS_WRITEM_REGVAL_POS = _TCPModbusClient.ARGS_WRITEM_REGVAL_POS
prepare_msg_query = _TCPModbusClient.prepare_msg_query
prepare_msg_read = _TCPModbusClient.prepare_msg_read
prepare_msg_write = _TCPModbusClient.prepare_msg_write
prepare_msg_writem = _TCPModbusClient.prepare_msg_writem
execute_command = _TCPModbusClient.execute_command
print_received_msg = _TCPModbusClient.print_received_msg
dev_query = _TCPModbusClient.dev_query
dev_read = _TCPModbusClient.dev_read
dev_write = _TCPModbusClient.dev_write
dev_writem = _TCPModbusClient.dev_writem
get_val = _TCPModbusClient.get_val
SUCCESS = _TCPModbusClient.SUCCESS
FAIL = _TCPModbusClient.FAIL
CRC16_SIZE = _TCPModbusClient.CRC16_SIZE
MODBUS_FUNC_READ_REG = _TCPModbusClient.MODBUS_FUNC_READ_REG
MODBUS_FUNC_WRITE_REG = _TCPModbusClient.MODBUS_FUNC_WRITE_REG
MODBUS_FUNC_WRITE_MULTIREG = _TCPModbusClient.MODBUS_FUNC_WRITE_MULTIREG
MODBUS_FUNC_REPORT_SLAVEID = _TCPModbusClient.MODBUS_FUNC_REPORT_SLAVEID
MODBUS_ERR_READ_REG = _TCPModbusClient.MODBUS_ERR_READ_REG
MODBUS_ERR_WRITE_REG = _TCPModbusClient.MODBUS_ERR_WRITE_REG
MODBUS_ERR_WRITE_MULTIREG = _TCPModbusClient.MODBUS_ERR_WRITE_MULTIREG
MODBUS_ERR_REPORT_SLAVEID = _TCPModbusClient.MODBUS_ERR_REPORT_SLAVEID
BYTEPOS_MODBUS_ADDR = _TCPModbusClient.BYTEPOS_MODBUS_ADDR
BYTEPOS_MODBUS_FUNC = _TCPModbusClient.BYTEPOS_MODBUS_FUNC
BYTEPOS_MODBUS_EXCEPTION_CODE = _TCPModbusClient.BYTEPOS_MODBUS_EXCEPTION_CODE
MODBUS_REG_READ_QTY_DEFAULT = _TCPModbusClient.MODBUS_REG_READ_QTY_DEFAULT
MODBUS_REG_READ_QTY_MIN = _TCPModbusClient.MODBUS_REG_READ_QTY_MIN
MODBUS_REG_READ_QTY_MAX = _TCPModbusClient.MODBUS_REG_READ_QTY_MAX
MODBUS_RUN_INDICATOR_ON = _TCPModbusClient.MODBUS_RUN_INDICATOR_ON
MODBUS_RUN_INDICATOR_OFF = _TCPModbusClient.MODBUS_RUN_INDICATOR_OFF
class modbus_req_read_reg(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_req_read_reg, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_req_read_reg, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_req_read_reg_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_req_read_reg_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_req_read_reg_modbus_addr_get, _TCPModbusClient.modbus_req_read_reg_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_req_read_reg_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_req_read_reg_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_req_read_reg_modbus_func_get, _TCPModbusClient.modbus_req_read_reg_modbus_func_set)
    __swig_setmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_req_read_reg_modbus_reg_addr_set
    __swig_getmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_req_read_reg_modbus_reg_addr_get
    if _newclass:modbus_reg_addr = _swig_property(_TCPModbusClient.modbus_req_read_reg_modbus_reg_addr_get, _TCPModbusClient.modbus_req_read_reg_modbus_reg_addr_set)
    __swig_setmethods__["modbus_reg_qty"] = _TCPModbusClient.modbus_req_read_reg_modbus_reg_qty_set
    __swig_getmethods__["modbus_reg_qty"] = _TCPModbusClient.modbus_req_read_reg_modbus_reg_qty_get
    if _newclass:modbus_reg_qty = _swig_property(_TCPModbusClient.modbus_req_read_reg_modbus_reg_qty_get, _TCPModbusClient.modbus_req_read_reg_modbus_reg_qty_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_req_read_reg(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_req_read_reg
    __del__ = lambda self : None;
modbus_req_read_reg_swigregister = _TCPModbusClient.modbus_req_read_reg_swigregister
modbus_req_read_reg_swigregister(modbus_req_read_reg)

class modbus_req_write_reg(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_req_write_reg, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_req_write_reg, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_req_write_reg_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_req_write_reg_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_req_write_reg_modbus_addr_get, _TCPModbusClient.modbus_req_write_reg_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_req_write_reg_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_req_write_reg_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_req_write_reg_modbus_func_get, _TCPModbusClient.modbus_req_write_reg_modbus_func_set)
    __swig_setmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_req_write_reg_modbus_reg_addr_set
    __swig_getmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_req_write_reg_modbus_reg_addr_get
    if _newclass:modbus_reg_addr = _swig_property(_TCPModbusClient.modbus_req_write_reg_modbus_reg_addr_get, _TCPModbusClient.modbus_req_write_reg_modbus_reg_addr_set)
    __swig_setmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_req_write_reg_modbus_reg_val_set
    __swig_getmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_req_write_reg_modbus_reg_val_get
    if _newclass:modbus_reg_val = _swig_property(_TCPModbusClient.modbus_req_write_reg_modbus_reg_val_get, _TCPModbusClient.modbus_req_write_reg_modbus_reg_val_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_req_write_reg(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_req_write_reg
    __del__ = lambda self : None;
modbus_req_write_reg_swigregister = _TCPModbusClient.modbus_req_write_reg_swigregister
modbus_req_write_reg_swigregister(modbus_req_write_reg)

class modbus_req_write_multireg(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_req_write_multireg, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_req_write_multireg, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_req_write_multireg_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_req_write_multireg_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_req_write_multireg_modbus_addr_get, _TCPModbusClient.modbus_req_write_multireg_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_req_write_multireg_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_req_write_multireg_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_req_write_multireg_modbus_func_get, _TCPModbusClient.modbus_req_write_multireg_modbus_func_set)
    __swig_setmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_req_write_multireg_modbus_reg_addr_set
    __swig_getmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_req_write_multireg_modbus_reg_addr_get
    if _newclass:modbus_reg_addr = _swig_property(_TCPModbusClient.modbus_req_write_multireg_modbus_reg_addr_get, _TCPModbusClient.modbus_req_write_multireg_modbus_reg_addr_set)
    __swig_setmethods__["modbus_reg_qty"] = _TCPModbusClient.modbus_req_write_multireg_modbus_reg_qty_set
    __swig_getmethods__["modbus_reg_qty"] = _TCPModbusClient.modbus_req_write_multireg_modbus_reg_qty_get
    if _newclass:modbus_reg_qty = _swig_property(_TCPModbusClient.modbus_req_write_multireg_modbus_reg_qty_get, _TCPModbusClient.modbus_req_write_multireg_modbus_reg_qty_set)
    __swig_setmethods__["modbus_val_bytes"] = _TCPModbusClient.modbus_req_write_multireg_modbus_val_bytes_set
    __swig_getmethods__["modbus_val_bytes"] = _TCPModbusClient.modbus_req_write_multireg_modbus_val_bytes_get
    if _newclass:modbus_val_bytes = _swig_property(_TCPModbusClient.modbus_req_write_multireg_modbus_val_bytes_get, _TCPModbusClient.modbus_req_write_multireg_modbus_val_bytes_set)
    __swig_setmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_req_write_multireg_modbus_reg_val_set
    __swig_getmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_req_write_multireg_modbus_reg_val_get
    if _newclass:modbus_reg_val = _swig_property(_TCPModbusClient.modbus_req_write_multireg_modbus_reg_val_get, _TCPModbusClient.modbus_req_write_multireg_modbus_reg_val_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_req_write_multireg(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_req_write_multireg
    __del__ = lambda self : None;
modbus_req_write_multireg_swigregister = _TCPModbusClient.modbus_req_write_multireg_swigregister
modbus_req_write_multireg_swigregister(modbus_req_write_multireg)

class modbus_req_report_slaveid(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_req_report_slaveid, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_req_report_slaveid, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_req_report_slaveid_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_req_report_slaveid_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_req_report_slaveid_modbus_addr_get, _TCPModbusClient.modbus_req_report_slaveid_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_req_report_slaveid_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_req_report_slaveid_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_req_report_slaveid_modbus_func_get, _TCPModbusClient.modbus_req_report_slaveid_modbus_func_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_req_report_slaveid(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_req_report_slaveid
    __del__ = lambda self : None;
modbus_req_report_slaveid_swigregister = _TCPModbusClient.modbus_req_report_slaveid_swigregister
modbus_req_report_slaveid_swigregister(modbus_req_report_slaveid)

class modbus_reply_read_reg(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_reply_read_reg, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_reply_read_reg, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_read_reg_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_read_reg_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_reply_read_reg_modbus_addr_get, _TCPModbusClient.modbus_reply_read_reg_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_read_reg_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_read_reg_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_reply_read_reg_modbus_func_get, _TCPModbusClient.modbus_reply_read_reg_modbus_func_set)
    __swig_setmethods__["modbus_val_bytes"] = _TCPModbusClient.modbus_reply_read_reg_modbus_val_bytes_set
    __swig_getmethods__["modbus_val_bytes"] = _TCPModbusClient.modbus_reply_read_reg_modbus_val_bytes_get
    if _newclass:modbus_val_bytes = _swig_property(_TCPModbusClient.modbus_reply_read_reg_modbus_val_bytes_get, _TCPModbusClient.modbus_reply_read_reg_modbus_val_bytes_set)
    __swig_setmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_reply_read_reg_modbus_reg_val_set
    __swig_getmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_reply_read_reg_modbus_reg_val_get
    if _newclass:modbus_reg_val = _swig_property(_TCPModbusClient.modbus_reply_read_reg_modbus_reg_val_get, _TCPModbusClient.modbus_reply_read_reg_modbus_reg_val_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_reply_read_reg(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_reply_read_reg
    __del__ = lambda self : None;
modbus_reply_read_reg_swigregister = _TCPModbusClient.modbus_reply_read_reg_swigregister
modbus_reply_read_reg_swigregister(modbus_reply_read_reg)

class modbus_reply_write_reg(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_reply_write_reg, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_reply_write_reg, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_write_reg_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_write_reg_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_reply_write_reg_modbus_addr_get, _TCPModbusClient.modbus_reply_write_reg_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_write_reg_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_write_reg_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_reply_write_reg_modbus_func_get, _TCPModbusClient.modbus_reply_write_reg_modbus_func_set)
    __swig_setmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_reply_write_reg_modbus_reg_addr_set
    __swig_getmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_reply_write_reg_modbus_reg_addr_get
    if _newclass:modbus_reg_addr = _swig_property(_TCPModbusClient.modbus_reply_write_reg_modbus_reg_addr_get, _TCPModbusClient.modbus_reply_write_reg_modbus_reg_addr_set)
    __swig_setmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_reply_write_reg_modbus_reg_val_set
    __swig_getmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_reply_write_reg_modbus_reg_val_get
    if _newclass:modbus_reg_val = _swig_property(_TCPModbusClient.modbus_reply_write_reg_modbus_reg_val_get, _TCPModbusClient.modbus_reply_write_reg_modbus_reg_val_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_reply_write_reg(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_reply_write_reg
    __del__ = lambda self : None;
modbus_reply_write_reg_swigregister = _TCPModbusClient.modbus_reply_write_reg_swigregister
modbus_reply_write_reg_swigregister(modbus_reply_write_reg)

class modbus_reply_write_multireg(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_reply_write_multireg, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_reply_write_multireg, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_write_multireg_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_write_multireg_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_reply_write_multireg_modbus_addr_get, _TCPModbusClient.modbus_reply_write_multireg_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_write_multireg_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_write_multireg_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_reply_write_multireg_modbus_func_get, _TCPModbusClient.modbus_reply_write_multireg_modbus_func_set)
    __swig_setmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_reply_write_multireg_modbus_reg_addr_set
    __swig_getmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_reply_write_multireg_modbus_reg_addr_get
    if _newclass:modbus_reg_addr = _swig_property(_TCPModbusClient.modbus_reply_write_multireg_modbus_reg_addr_get, _TCPModbusClient.modbus_reply_write_multireg_modbus_reg_addr_set)
    __swig_setmethods__["modbus_reg_qty"] = _TCPModbusClient.modbus_reply_write_multireg_modbus_reg_qty_set
    __swig_getmethods__["modbus_reg_qty"] = _TCPModbusClient.modbus_reply_write_multireg_modbus_reg_qty_get
    if _newclass:modbus_reg_qty = _swig_property(_TCPModbusClient.modbus_reply_write_multireg_modbus_reg_qty_get, _TCPModbusClient.modbus_reply_write_multireg_modbus_reg_qty_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_reply_write_multireg(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_reply_write_multireg
    __del__ = lambda self : None;
modbus_reply_write_multireg_swigregister = _TCPModbusClient.modbus_reply_write_multireg_swigregister
modbus_reply_write_multireg_swigregister(modbus_reply_write_multireg)

class modbus_reply_report_slaveid(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_reply_report_slaveid, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_reply_report_slaveid, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_reply_report_slaveid_modbus_addr_get, _TCPModbusClient.modbus_reply_report_slaveid_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_reply_report_slaveid_modbus_func_get, _TCPModbusClient.modbus_reply_report_slaveid_modbus_func_set)
    __swig_setmethods__["modbus_val_bytes"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_val_bytes_set
    __swig_getmethods__["modbus_val_bytes"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_val_bytes_get
    if _newclass:modbus_val_bytes = _swig_property(_TCPModbusClient.modbus_reply_report_slaveid_modbus_val_bytes_get, _TCPModbusClient.modbus_reply_report_slaveid_modbus_val_bytes_set)
    __swig_setmethods__["modbus_slaveid"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_slaveid_set
    __swig_getmethods__["modbus_slaveid"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_slaveid_get
    if _newclass:modbus_slaveid = _swig_property(_TCPModbusClient.modbus_reply_report_slaveid_modbus_slaveid_get, _TCPModbusClient.modbus_reply_report_slaveid_modbus_slaveid_set)
    __swig_setmethods__["modbus_run_indicator"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_run_indicator_set
    __swig_getmethods__["modbus_run_indicator"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_run_indicator_get
    if _newclass:modbus_run_indicator = _swig_property(_TCPModbusClient.modbus_reply_report_slaveid_modbus_run_indicator_get, _TCPModbusClient.modbus_reply_report_slaveid_modbus_run_indicator_set)
    __swig_setmethods__["modbus_additional"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_additional_set
    __swig_getmethods__["modbus_additional"] = _TCPModbusClient.modbus_reply_report_slaveid_modbus_additional_get
    if _newclass:modbus_additional = _swig_property(_TCPModbusClient.modbus_reply_report_slaveid_modbus_additional_get, _TCPModbusClient.modbus_reply_report_slaveid_modbus_additional_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_reply_report_slaveid(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_reply_report_slaveid
    __del__ = lambda self : None;
modbus_reply_report_slaveid_swigregister = _TCPModbusClient.modbus_reply_report_slaveid_swigregister
modbus_reply_report_slaveid_swigregister(modbus_reply_report_slaveid)

MAX_REG_VAL = _TCPModbusClient.MAX_REG_VAL
MODBUS_MIN_REPLY = _TCPModbusClient.MODBUS_MIN_REPLY
class modbus_reply(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, modbus_reply, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, modbus_reply, name)
    __repr__ = _swig_repr
    __swig_setmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_modbus_addr_set
    __swig_getmethods__["modbus_addr"] = _TCPModbusClient.modbus_reply_modbus_addr_get
    if _newclass:modbus_addr = _swig_property(_TCPModbusClient.modbus_reply_modbus_addr_get, _TCPModbusClient.modbus_reply_modbus_addr_set)
    __swig_setmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_modbus_func_set
    __swig_getmethods__["modbus_func"] = _TCPModbusClient.modbus_reply_modbus_func_get
    if _newclass:modbus_func = _swig_property(_TCPModbusClient.modbus_reply_modbus_func_get, _TCPModbusClient.modbus_reply_modbus_func_set)
    __swig_setmethods__["modbus_val_bytes"] = _TCPModbusClient.modbus_reply_modbus_val_bytes_set
    __swig_getmethods__["modbus_val_bytes"] = _TCPModbusClient.modbus_reply_modbus_val_bytes_get
    if _newclass:modbus_val_bytes = _swig_property(_TCPModbusClient.modbus_reply_modbus_val_bytes_get, _TCPModbusClient.modbus_reply_modbus_val_bytes_set)
    __swig_setmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_reply_modbus_reg_addr_set
    __swig_getmethods__["modbus_reg_addr"] = _TCPModbusClient.modbus_reply_modbus_reg_addr_get
    if _newclass:modbus_reg_addr = _swig_property(_TCPModbusClient.modbus_reply_modbus_reg_addr_get, _TCPModbusClient.modbus_reply_modbus_reg_addr_set)
    __swig_setmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_reply_modbus_reg_val_set
    __swig_getmethods__["modbus_reg_val"] = _TCPModbusClient.modbus_reply_modbus_reg_val_get
    if _newclass:modbus_reg_val = _swig_property(_TCPModbusClient.modbus_reply_modbus_reg_val_get, _TCPModbusClient.modbus_reply_modbus_reg_val_set)
    __swig_setmethods__["modbus_reg_qty"] = _TCPModbusClient.modbus_reply_modbus_reg_qty_set
    __swig_getmethods__["modbus_reg_qty"] = _TCPModbusClient.modbus_reply_modbus_reg_qty_get
    if _newclass:modbus_reg_qty = _swig_property(_TCPModbusClient.modbus_reply_modbus_reg_qty_get, _TCPModbusClient.modbus_reply_modbus_reg_qty_set)
    __swig_setmethods__["crc"] = _TCPModbusClient.modbus_reply_crc_set
    __swig_getmethods__["crc"] = _TCPModbusClient.modbus_reply_crc_get
    if _newclass:crc = _swig_property(_TCPModbusClient.modbus_reply_crc_get, _TCPModbusClient.modbus_reply_crc_set)
    __swig_setmethods__["modbus_slaveid"] = _TCPModbusClient.modbus_reply_modbus_slaveid_set
    __swig_getmethods__["modbus_slaveid"] = _TCPModbusClient.modbus_reply_modbus_slaveid_get
    if _newclass:modbus_slaveid = _swig_property(_TCPModbusClient.modbus_reply_modbus_slaveid_get, _TCPModbusClient.modbus_reply_modbus_slaveid_set)
    __swig_setmethods__["modbus_run_indicator"] = _TCPModbusClient.modbus_reply_modbus_run_indicator_set
    __swig_getmethods__["modbus_run_indicator"] = _TCPModbusClient.modbus_reply_modbus_run_indicator_get
    if _newclass:modbus_run_indicator = _swig_property(_TCPModbusClient.modbus_reply_modbus_run_indicator_get, _TCPModbusClient.modbus_reply_modbus_run_indicator_set)
    __swig_setmethods__["modbus_additional"] = _TCPModbusClient.modbus_reply_modbus_additional_set
    __swig_getmethods__["modbus_additional"] = _TCPModbusClient.modbus_reply_modbus_additional_get
    if _newclass:modbus_additional = _swig_property(_TCPModbusClient.modbus_reply_modbus_additional_get, _TCPModbusClient.modbus_reply_modbus_additional_set)
    def __init__(self, *args): 
        this = _TCPModbusClient.new_modbus_reply(*args)
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _TCPModbusClient.delete_modbus_reply
    __del__ = lambda self : None;
modbus_reply_swigregister = _TCPModbusClient.modbus_reply_swigregister
modbus_reply_swigregister(modbus_reply)

DieWithError = _TCPModbusClient.DieWithError
calc_crc16 = _TCPModbusClient.calc_crc16
read_crc16 = _TCPModbusClient.read_crc16
print_modbus_reply_read_reg = _TCPModbusClient.print_modbus_reply_read_reg
print_modbus_reply_write_reg = _TCPModbusClient.print_modbus_reply_write_reg
print_modbus_reply_write_multireg = _TCPModbusClient.print_modbus_reply_write_multireg
print_modbus_reply_report_slaveid = _TCPModbusClient.print_modbus_reply_report_slaveid





"""
Copyright (c) 2011, 2012, Regents of the University of California
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions 
are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
"""
"""
@author Stephen Dawson-Haggerty <stevedh@eecs.berkeley.edu>
"""

import os
import logging
import shelve
import pickle
import numpy as np
import unittest

CACHEDIR='.cache'

def from_key(s):
    x = s.split('-')
    return (int(x[0]), int(x[1]))

def filter_data(range, data):
    idx = (data[:,0] >= range[0]) & (data[:,0] <= range[1])
    return data[np.nonzero(idx)]

class TimeseriesCache:
    """Cache of timeseries data.

    A time series cache allows clients to store on disk a single
    time-series (a sequence of time, value tuples) and read back
    ranges.  If only part of the data is available, the cache will
    return the segments so the application can query the data store
    for the remaining data, and presumably insert it into the cache.
    """
    def __init__(self, identifier, ondisk=True):
        self.log = logging.getLogger("TimeseriesCache")
        if ondisk:
            try:
                os.makedirs(CACHEDIR)
            except OSError:
                pass
            self.cache = shelve.open(os.path.join(CACHEDIR, identifier), 
                                     protocol=2)
        else:
            self.log.info("Using non-persistant cache")
            self.cache = {}

    def close(self):
        self.cache.close()

    def clear(self):
        self.cache.clear()
        self.cache.sync()

    def insert(self, substream, start, end, data):
        """Insert new data into the cache
        """
        key = "%i-%i" % (start, end)
        substream = str(substream)
        if len(data) == 0: return
        if not self.cache.has_key(substream):
            self.cache[substream] = {key:  data}
        else:
            # do this due to copy issues with shelve
            ssdata = self.cache[substream]
            ssdata[key] = data
            self.cache[substream] = ssdata
        self.cache.sync()

    def set_meta(self, meta):
        self.cache['meta'] = meta
        self.cache.sync()

    def get_meta(self):
        return self.cache.get('meta')

    def read(self, substream, start, end):
        """Read back fragments of data from the cache in the range [start, end] (inclusive)
        """
        pointer = 0
        substream = str(substream)
        rv = []
        if not substream in self.cache:
            return []

        for k in sorted(self.cache[substream].keys(), key=lambda k: from_key(k)[0]):
            (s,e) = from_key(k)
            key = None
            if s <= start and e > start:
                key = (max(pointer, start), min(e, end))
            elif s >= start and s < end:
                key = (max(pointer, s), min(e, end))

            if key != None and key[1] > pointer:
                pointer = key[1]
                rv.append((key, filter_data(key, self.cache[substream][k])))
        return rv

class TimeseriesTest(unittest.TestCase):
    SS = 0

    def setUp(self):
        c = TimeseriesCache('test')
        c.insert(self.SS, 0, 10, np.array(zip(range(0, 10), range(0,10))))
        c.insert(self.SS, 20, 30, np.array(zip(range(20, 30), range(20,30))))
        c.insert(self.SS, 30, 40, np.array(zip(range(30, 40), range(30,40))))
        c.close()

    def tearDone(self):
        c = TimeseriesCache('test')
        c.clear()
        c.close()

    def testInserted(self):
        c = TimeseriesCache('test')
        self.assertTrue(c.cache.has_key('0'))
        self.assertTrue(c.cache['0'].has_key('0-10'))
        self.assertTrue(c.cache['0'].has_key('20-30'))

    def testSave(self):
        c = TimeseriesCache('test')
        c.set_meta({'test': True})
        c.close()
        r = TimeseriesCache('test')
        self.assertEqual(r.get_meta(), {'test' : True})
        r.close()

    def testSimpleRead(self):
        c = TimeseriesCache('test')
        rv = c.read(self.SS, 0, 1)
        self.assertEqual(len(rv),1)
        rv = rv[0]
        self.assertEqual(rv[0], (0,1))
        self.assertEqual(len(rv[1]), 2)
        c.close()
        
    def testPartialRead(self):
        c = TimeseriesCache('test')
        rv = c.read(self.SS, 5, 15)
        self.assertEqual(len(rv), 1)
        rv = rv[0]
        self.assertEqual(rv[0], (5, 10))
        self.assertEqual(len(rv[1]), 5)
        self.assertEqual(rv[1][0,0], 5)
        self.assertEqual(rv[1][-1,0], 9)
        c.close()

    def testDoubleSplitRead(self):
        c = TimeseriesCache('test')
        rv = c.read(self.SS, 5, 25)
        self.assertEqual(len(rv), 2)
        # first frag
        self.assertEqual(rv[0][0], (5,10))
        self.assertEqual(rv[0][1][0,0], 5)
        self.assertEqual(rv[0][1][-1,0], 9)

        self.assertEqual(rv[1][0], (20,25))
        self.assertEqual(rv[1][1][0,0], 20)
        self.assertEqual(rv[1][1][-1,0], 25)
        c.close()

    def testCoveringRead(self):
        c = TimeseriesCache('test')
        rv = c.read(self.SS, 5, 30)
        self.assertEqual(len(rv), 2)
        self.assertEqual(rv[0][0], (5,10))
        self.assertEqual(rv[0][1][0,0], 5)
        self.assertEqual(rv[0][1][-1,0], 9)

        self.assertEqual(rv[1][0], (20,30))
        self.assertEqual(rv[1][1][0,0], 20)
        self.assertEqual(rv[1][1][-1,0], 29)
        c.close()

    def testNeighboringRead(self):
        c = TimeseriesCache('test')
        rv = c.read(self.SS, 15, 44)
        c.close()

    def testNoSS(self):
        c = TimeseriesCache('test')
        self.assertEqual(c.read(self.SS+1, 0, 10), [])
        c.close()

if __name__ == '__main__':
    unittest.main()



"""
Copyright (c) 2011, 2012, Regents of the University of California
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions 
are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
"""
"""sMAP source for Vaisala WXT520 weather station running in automatic
ASCII mode.  It should be connected via ethernet using a bridge from
the SDI-12 connection.  The driver connects to the address specified
and reads lines from it, adding the data to the sMAP driver.

Parameters:
"Address" : ip address or host name of device
"Port" [default 4660] : tcp port to connect on.


@author Stephen Dawson-Haggerty <stevedh@eecs.berkeley.edu>
"""

import sys
import socket
import time
import logging

from twisted.internet import reactor
from twisted.internet.protocol import ReconnectingClientFactory
from twisted.internet.endpoints import TCP4ClientEndpoint
from twisted.protocols.basic import LineReceiver

from smap.driver import SmapDriver

VAISALA_UNITS = {
    'R1' : {
      'D' : 'deg',
      'M' : 'm/s',
      },
    'R2' : {
      'C' : 'C',
      'P' : 'rh',
      'H' : 'Pa',
      },
    'R3' : {
      'M' : 'mm',
      's' : 'second',
      }
    }

VAISALA_POINTS = {
    'R1' : (
       'wind', {
         'direction' : ('Dm', 'Dn', 'Dx'),
         'speed' : ('Sm', 'Sn', 'Sx')
         }
       ),
    'R2' : (
       'pth', {
         'temperature' : ('Ta', None, None),
         'rh' : ('Ua', None, None),
         'pressure' : ('Pa', None, None),
         }
       ),
    'R3' : (
       'precipitation', {
          'rain_accumulation' : ('Rc', None, None),
          'rain_duration' : ('Rd', None, None),
          'rain_intensity' : ('Ri', None, 'Rp'),
          'hail_accumulation' : ('Hc', None, None),
          'hail_duration' : ('Hd', None, None),
          'hail_intensity' : ('Hi', None, 'Hp')
          }
       )
    }

class VaisalaDriver(SmapDriver):
    class VaisalaListener(LineReceiver):
        def lineReceived(self, line):
            self.log.debug("Read: " + line)
            try:
                self.process(line)
            except Exception, e:
                self.log.error("Error in update: " + str(e))

        def process(self, line):
            fields = line.split(',')
            reg = fields[0][1:]
            def proc_field(f):
                v = f.split('=')
                return (v[0], (v[1][:-1], v[1][-1]))
            data = dict(map(proc_field, fields[1:]))

            if VAISALA_POINTS.has_key(reg):
                ts = int(time.time())
                point = VAISALA_POINTS[reg][0]
                # create the point in the smap tree if necessary
                for k,v in VAISALA_POINTS[reg][1].iteritems():
                    unit = VAISALA_UNITS[reg][data[v[0]][1]]
                    path = '/%s/%s' % (point, k)
                    if not self.inst.lookup(path):
                        self.inst.add_timeseries(path, unit, data_type='double')

                    val_ = float(data.get(v[0])[0])
                    min_ = data.get(v[1], None)
                    if min_ != None: min_ = min_[0]
                    max_ = data.get(v[2], None)
                    if max_ != None: max_ = max_[0]
                    self.inst.add(path, ts, val_)


    def setup(self, opts):
        self.host = opts.get('Address')
        self.port = int(opts.get('Port', 4660))
        self.log = logging.getLogger('VaisalaReader')
        self.set_metadata('/', {
            'Extra/Driver' : 'smap.drivers.vaisala.VaisalaDriver',
            'Instrument/Manufacturer' : 'Vaisala',
            'Instrument/Model' : 'WXT520' })
            

    def gotprotocol(self, p):
        # give the new listener class references to us
        p.inst = self
        p.log = self.log

    def start(self):
        # SDH : this is supposed to reconnect us automatically when
        # the socket dies.  I haven't tried it, though.
        self.factory = ReconnectingClientFactory()
        self.factory.protocol = VaisalaDriver.VaisalaListener
        self.point = TCP4ClientEndpoint(reactor, self.host, self.port)
        d = self.point.connect(self.factory)
        d.addCallback(self.gotprotocol)




#
# This class is automatically generated by mig. DO NOT EDIT THIS FILE.
# This class implements a Python interface to the 'AcReport'
# message type.
#

import tinyos.message.Message

# The default size of this message type in bytes.
DEFAULT_MESSAGE_SIZE = 61

# The Active Message type associated with this message.
AM_TYPE = -1

class AcReport(tinyos.message.Message.Message):
    # Create a new AcReport of size 61.
    def __init__(self, data="", addr=None, gid=None, base_offset=0, data_length=61):
        tinyos.message.Message.Message.__init__(self, data, addr, gid, base_offset, data_length)
        self.amTypeSet(AM_TYPE)
    
    # Get AM_TYPE
    def get_amType(cls):
        return AM_TYPE
    
    get_amType = classmethod(get_amType)
    
    #
    # Return a String representation of this message. Includes the
    # message type name and the non-indexed field values.
    #
    def __str__(self):
        s = "Message <AcReport> \n"
        try:
            s += "  [eui64=";
            for i in range(0, 8):
                s += "0x%x " % (self.getElement_eui64(i) & 0xff)
            s += "]\n";
        except:
            pass
        try:
            s += "  [route.hop_limit=0x%x]\n" % (self.get_route_hop_limit())
        except:
            pass
        try:
            s += "  [route.parent=0x%x]\n" % (self.get_route_parent())
        except:
            pass
        try:
            s += "  [route.parent_metric=0x%x]\n" % (self.get_route_parent_metric())
        except:
            pass
        try:
            s += "  [route.parent_etx=0x%x]\n" % (self.get_route_parent_etx())
        except:
            pass
        try:
            s += "  [seq=0x%x]\n" % (self.get_seq())
        except:
            pass
        try:
            s += "  [localTime=0x%x]\n" % (self.get_localTime())
        except:
            pass
        try:
            s += "  [globalTime=0x%x]\n" % (self.get_globalTime())
        except:
            pass
        try:
            s += "  [period=0x%x]\n" % (self.get_period())
        except:
            pass
        try:
            s += "  [readings.cumulativeRealEnergy=";
            for i in range(0, 2):
                s += "0x%x " % (self.getElement_readings_cumulativeRealEnergy(i))
            s += "]\n";
        except:
            pass
        try:
            s += "  [readings.averageRealPower=";
            for i in range(0, 2):
                s += "0x%x " % (self.getElement_readings_averageRealPower(i) & 0xff)
            s += "]\n";
        except:
            pass
        try:
            s += "  [readings.averageApparentPower=";
            for i in range(0, 2):
                s += "0x%x " % (self.getElement_readings_averageApparentPower(i) & 0xff)
            s += "]\n";
        except:
            pass
        return s

    # Message-type-specific access methods appear below.

    #
    # Accessor methods for field: eui64
    #   Field type: short[]
    #   Offset (bits): 0
    #   Size of each element (bits): 8
    #

    #
    # Return whether the field 'eui64' is signed (False).
    #
    def isSigned_eui64(self):
        return False
    
    #
    # Return whether the field 'eui64' is an array (True).
    #
    def isArray_eui64(self):
        return True
    
    #
    # Return the offset (in bytes) of the field 'eui64'
    #
    def offset_eui64(self, index1):
        offset = 0
        if index1 < 0 or index1 >= 8:
            raise IndexError
        offset += 0 + index1 * 8
        return (offset / 8)
    
    #
    # Return the offset (in bits) of the field 'eui64'
    #
    def offsetBits_eui64(self, index1):
        offset = 0
        if index1 < 0 or index1 >= 8:
            raise IndexError
        offset += 0 + index1 * 8
        return offset
    
    #
    # Return the entire array 'eui64' as a short[]
    #
    def get_eui64(self):
        tmp = [None]*8
        for index0 in range (0, self.numElements_eui64(0)):
                tmp[index0] = self.getElement_eui64(index0)
        return tmp
    
    #
    # Set the contents of the array 'eui64' from the given short[]
    #
    def set_eui64(self, value):
        for index0 in range(0, len(value)):
            self.setElement_eui64(index0, value[index0])

    #
    # Return an element (as a short) of the array 'eui64'
    #
    def getElement_eui64(self, index1):
        return self.getUIntElement(self.offsetBits_eui64(index1), 8, 1)
    
    #
    # Set an element of the array 'eui64'
    #
    def setElement_eui64(self, index1, value):
        self.setUIntElement(self.offsetBits_eui64(index1), 8, value, 1)
    
    #
    # Return the total size, in bytes, of the array 'eui64'
    #
    def totalSize_eui64(self):
        return (64 / 8)
    
    #
    # Return the total size, in bits, of the array 'eui64'
    #
    def totalSizeBits_eui64(self):
        return 64
    
    #
    # Return the size, in bytes, of each element of the array 'eui64'
    #
    def elementSize_eui64(self):
        return (8 / 8)
    
    #
    # Return the size, in bits, of each element of the array 'eui64'
    #
    def elementSizeBits_eui64(self):
        return 8
    
    #
    # Return the number of dimensions in the array 'eui64'
    #
    def numDimensions_eui64(self):
        return 1
    
    #
    # Return the number of elements in the array 'eui64'
    #
    def numElements_eui64():
        return 8
    
    #
    # Return the number of elements in the array 'eui64'
    # for the given dimension.
    #
    def numElements_eui64(self, dimension):
        array_dims = [ 8,  ]
        if dimension < 0 or dimension >= 1:
            raise IndexException
        if array_dims[dimension] == 0:
            raise IndexError
        return array_dims[dimension]
    
    #
    # Fill in the array 'eui64' with a String
    #
    def setString_eui64(self, s):
         l = len(s)
         for i in range(0, l):
             self.setElement_eui64(i, ord(s[i]));
         self.setElement_eui64(l, 0) #null terminate
    
    #
    # Read the array 'eui64' as a String
    #
    def getString_eui64(self):
        carr = "";
        for i in range(0, 4000):
            if self.getElement_eui64(i) == chr(0):
                break
            carr += self.getElement_eui64(i)
        return carr
    
    #
    # Accessor methods for field: route.hop_limit
    #   Field type: short
    #   Offset (bits): 64
    #   Size (bits): 8
    #

    #
    # Return whether the field 'route.hop_limit' is signed (False).
    #
    def isSigned_route_hop_limit(self):
        return False
    
    #
    # Return whether the field 'route.hop_limit' is an array (False).
    #
    def isArray_route_hop_limit(self):
        return False
    
    #
    # Return the offset (in bytes) of the field 'route.hop_limit'
    #
    def offset_route_hop_limit(self):
        return (64 / 8)
    
    #
    # Return the offset (in bits) of the field 'route.hop_limit'
    #
    def offsetBits_route_hop_limit(self):
        return 64
    
    #
    # Return the value (as a short) of the field 'route.hop_limit'
    #
    def get_route_hop_limit(self):
        return self.getUIntElement(self.offsetBits_route_hop_limit(), 8, 1)
    
    #
    # Set the value of the field 'route.hop_limit'
    #
    def set_route_hop_limit(self, value):
        self.setUIntElement(self.offsetBits_route_hop_limit(), 8, value, 1)
    
    #
    # Return the size, in bytes, of the field 'route.hop_limit'
    #
    def size_route_hop_limit(self):
        return (8 / 8)
    
    #
    # Return the size, in bits, of the field 'route.hop_limit'
    #
    def sizeBits_route_hop_limit(self):
        return 8
    
    #
    # Accessor methods for field: route.parent
    #   Field type: int
    #   Offset (bits): 72
    #   Size (bits): 16
    #

    #
    # Return whether the field 'route.parent' is signed (False).
    #
    def isSigned_route_parent(self):
        return False
    
    #
    # Return whether the field 'route.parent' is an array (False).
    #
    def isArray_route_parent(self):
        return False
    
    #
    # Return the offset (in bytes) of the field 'route.parent'
    #
    def offset_route_parent(self):
        return (72 / 8)
    
    #
    # Return the offset (in bits) of the field 'route.parent'
    #
    def offsetBits_route_parent(self):
        return 72
    
    #
    # Return the value (as a int) of the field 'route.parent'
    #
    def get_route_parent(self):
        return self.getUIntElement(self.offsetBits_route_parent(), 16, 1)
    
    #
    # Set the value of the field 'route.parent'
    #
    def set_route_parent(self, value):
        self.setUIntElement(self.offsetBits_route_parent(), 16, value, 1)
    
    #
    # Return the size, in bytes, of the field 'route.parent'
    #
    def size_route_parent(self):
        return (16 / 8)
    
    #
    # Return the size, in bits, of the field 'route.parent'
    #
    def sizeBits_route_parent(self):
        return 16
    
    #
    # Accessor methods for field: route.parent_metric
    #   Field type: int
    #   Offset (bits): 88
    #   Size (bits): 16
    #

    #
    # Return whether the field 'route.parent_metric' is signed (False).
    #
    def isSigned_route_parent_metric(self):
        return False
    
    #
    # Return whether the field 'route.parent_metric' is an array (False).
    #
    def isArray_route_parent_metric(self):
        return False
    
    #
    # Return the offset (in bytes) of the field 'route.parent_metric'
    #
    def offset_route_parent_metric(self):
        return (88 / 8)
    
    #
    # Return the offset (in bits) of the field 'route.parent_metric'
    #
    def offsetBits_route_parent_metric(self):
        return 88
    
    #
    # Return the value (as a int) of the field 'route.parent_metric'
    #
    def get_route_parent_metric(self):
        return self.getUIntElement(self.offsetBits_route_parent_metric(), 16, 1)
    
    #
    # Set the value of the field 'route.parent_metric'
    #
    def set_route_parent_metric(self, value):
        self.setUIntElement(self.offsetBits_route_parent_metric(), 16, value, 1)
    
    #
    # Return the size, in bytes, of the field 'route.parent_metric'
    #
    def size_route_parent_metric(self):
        return (16 / 8)
    
    #
    # Return the size, in bits, of the field 'route.parent_metric'
    #
    def sizeBits_route_parent_metric(self):
        return 16
    
    #
    # Accessor methods for field: route.parent_etx
    #   Field type: int
    #   Offset (bits): 104
    #   Size (bits): 16
    #

    #
    # Return whether the field 'route.parent_etx' is signed (False).
    #
    def isSigned_route_parent_etx(self):
        return False
    
    #
    # Return whether the field 'route.parent_etx' is an array (False).
    #
    def isArray_route_parent_etx(self):
        return False
    
    #
    # Return the offset (in bytes) of the field 'route.parent_etx'
    #
    def offset_route_parent_etx(self):
        return (104 / 8)
    
    #
    # Return the offset (in bits) of the field 'route.parent_etx'
    #
    def offsetBits_route_parent_etx(self):
        return 104
    
    #
    # Return the value (as a int) of the field 'route.parent_etx'
    #
    def get_route_parent_etx(self):
        return self.getUIntElement(self.offsetBits_route_parent_etx(), 16, 1)
    
    #
    # Set the value of the field 'route.parent_etx'
    #
    def set_route_parent_etx(self, value):
        self.setUIntElement(self.offsetBits_route_parent_etx(), 16, value, 1)
    
    #
    # Return the size, in bytes, of the field 'route.parent_etx'
    #
    def size_route_parent_etx(self):
        return (16 / 8)
    
    #
    # Return the size, in bits, of the field 'route.parent_etx'
    #
    def sizeBits_route_parent_etx(self):
        return 16
    
    #
    # Accessor methods for field: seq
    #   Field type: long
    #   Offset (bits): 120
    #   Size (bits): 32
    #

    #
    # Return whether the field 'seq' is signed (False).
    #
    def isSigned_seq(self):
        return False
    
    #
    # Return whether the field 'seq' is an array (False).
    #
    def isArray_seq(self):
        return False
    
    #
    # Return the offset (in bytes) of the field 'seq'
    #
    def offset_seq(self):
        return (120 / 8)
    
    #
    # Return the offset (in bits) of the field 'seq'
    #
    def offsetBits_seq(self):
        return 120
    
    #
    # Return the value (as a long) of the field 'seq'
    #
    def get_seq(self):
        return self.getUIntElement(self.offsetBits_seq(), 32, 1)
    
    #
    # Set the value of the field 'seq'
    #
    def set_seq(self, value):
        self.setUIntElement(self.offsetBits_seq(), 32, value, 1)
    
    #
    # Return the size, in bytes, of the field 'seq'
    #
    def size_seq(self):
        return (32 / 8)
    
    #
    # Return the size, in bits, of the field 'seq'
    #
    def sizeBits_seq(self):
        return 32
    
    #
    # Accessor methods for field: localTime
    #   Field type: long
    #   Offset (bits): 152
    #   Size (bits): 32
    #

    #
    # Return whether the field 'localTime' is signed (False).
    #
    def isSigned_localTime(self):
        return False
    
    #
    # Return whether the field 'localTime' is an array (False).
    #
    def isArray_localTime(self):
        return False
    
    #
    # Return the offset (in bytes) of the field 'localTime'
    #
    def offset_localTime(self):
        return (152 / 8)
    
    #
    # Return the offset (in bits) of the field 'localTime'
    #
    def offsetBits_localTime(self):
        return 152
    
    #
    # Return the value (as a long) of the field 'localTime'
    #
    def get_localTime(self):
        return self.getUIntElement(self.offsetBits_localTime(), 32, 1)
    
    #
    # Set the value of the field 'localTime'
    #
    def set_localTime(self, value):
        self.setUIntElement(self.offsetBits_localTime(), 32, value, 1)
    
    #
    # Return the size, in bytes, of the field 'localTime'
    #
    def size_localTime(self):
        return (32 / 8)
    
    #
    # Return the size, in bits, of the field 'localTime'
    #
    def sizeBits_localTime(self):
        return 32
    
    #
    # Accessor methods for field: globalTime
    #   Field type: long
    #   Offset (bits): 184
    #   Size (bits): 32
    #

    #
    # Return whether the field 'globalTime' is signed (False).
    #
    def isSigned_globalTime(self):
        return False
    
    #
    # Return whether the field 'globalTime' is an array (False).
    #
    def isArray_globalTime(self):
        return False
    
    #
    # Return the offset (in bytes) of the field 'globalTime'
    #
    def offset_globalTime(self):
        return (184 / 8)
    
    #
    # Return the offset (in bits) of the field 'globalTime'
    #
    def offsetBits_globalTime(self):
        return 184
    
    #
    # Return the value (as a long) of the field 'globalTime'
    #
    def get_globalTime(self):
        return self.getUIntElement(self.offsetBits_globalTime(), 32, 1)
    
    #
    # Set the value of the field 'globalTime'
    #
    def set_globalTime(self, value):
        self.setUIntElement(self.offsetBits_globalTime(), 32, value, 1)
    
    #
    # Return the size, in bytes, of the field 'globalTime'
    #
    def size_globalTime(self):
        return (32 / 8)
    
    #
    # Return the size, in bits, of the field 'globalTime'
    #
    def sizeBits_globalTime(self):
        return 32
    
    #
    # Accessor methods for field: period
    #   Field type: int
    #   Offset (bits): 216
    #   Size (bits): 16
    #

    #
    # Return whether the field 'period' is signed (False).
    #
    def isSigned_period(self):
        return False
    
    #
    # Return whether the field 'period' is an array (False).
    #
    def isArray_period(self):
        return False
    
    #
    # Return the offset (in bytes) of the field 'period'
    #
    def offset_period(self):
        return (216 / 8)
    
    #
    # Return the offset (in bits) of the field 'period'
    #
    def offsetBits_period(self):
        return 216
    
    #
    # Return the value (as a int) of the field 'period'
    #
    def get_period(self):
        return self.getUIntElement(self.offsetBits_period(), 16, 1)
    
    #
    # Set the value of the field 'period'
    #
    def set_period(self, value):
        self.setUIntElement(self.offsetBits_period(), 16, value, 1)
    
    #
    # Return the size, in bytes, of the field 'period'
    #
    def size_period(self):
        return (16 / 8)
    
    #
    # Return the size, in bits, of the field 'period'
    #
    def sizeBits_period(self):
        return 16
    
    #
    # Accessor methods for field: readings.cumulativeRealEnergy
    #   Field type: long[]
    #   Offset (bits): 0
    #   Size of each element (bits): 64
    #

    #
    # Return whether the field 'readings.cumulativeRealEnergy' is signed (False).
    #
    def isSigned_readings_cumulativeRealEnergy(self):
        return False
    
    #
    # Return whether the field 'readings.cumulativeRealEnergy' is an array (True).
    #
    def isArray_readings_cumulativeRealEnergy(self):
        return True
    
    #
    # Return the offset (in bytes) of the field 'readings.cumulativeRealEnergy'
    #
    def offset_readings_cumulativeRealEnergy(self, index1):
        offset = 0
        if index1 < 0 or index1 >= 2:
            raise IndexError
        offset += 232 + index1 * 128
        return (offset / 8)
    
    #
    # Return the offset (in bits) of the field 'readings.cumulativeRealEnergy'
    #
    def offsetBits_readings_cumulativeRealEnergy(self, index1):
        offset = 0
        if index1 < 0 or index1 >= 2:
            raise IndexError
        offset += 232 + index1 * 128
        return offset
    
    #
    # Return the entire array 'readings.cumulativeRealEnergy' as a long[]
    #
    def get_readings_cumulativeRealEnergy(self):
        tmp = [None]*2
        for index0 in range (0, self.numElements_readings_cumulativeRealEnergy(0)):
                tmp[index0] = self.getElement_readings_cumulativeRealEnergy(index0)
        return tmp
    
    #
    # Set the contents of the array 'readings.cumulativeRealEnergy' from the given long[]
    #
    def set_readings_cumulativeRealEnergy(self, value):
        for index0 in range(0, len(value)):
            self.setElement_readings_cumulativeRealEnergy(index0, value[index0])

    #
    # Return an element (as a long) of the array 'readings.cumulativeRealEnergy'
    #
    def getElement_readings_cumulativeRealEnergy(self, index1):
        return self.getUIntElement(self.offsetBits_readings_cumulativeRealEnergy(index1), 64, 1)
    
    #
    # Set an element of the array 'readings.cumulativeRealEnergy'
    #
    def setElement_readings_cumulativeRealEnergy(self, index1, value):
        self.setUIntElement(self.offsetBits_readings_cumulativeRealEnergy(index1), 64, value, 1)
    
    #
    # Return the total size, in bytes, of the array 'readings.cumulativeRealEnergy'
    #
    def totalSize_readings_cumulativeRealEnergy(self):
        return (256 / 8)
    
    #
    # Return the total size, in bits, of the array 'readings.cumulativeRealEnergy'
    #
    def totalSizeBits_readings_cumulativeRealEnergy(self):
        return 256
    
    #
    # Return the size, in bytes, of each element of the array 'readings.cumulativeRealEnergy'
    #
    def elementSize_readings_cumulativeRealEnergy(self):
        return (64 / 8)
    
    #
    # Return the size, in bits, of each element of the array 'readings.cumulativeRealEnergy'
    #
    def elementSizeBits_readings_cumulativeRealEnergy(self):
        return 64
    
    #
    # Return the number of dimensions in the array 'readings.cumulativeRealEnergy'
    #
    def numDimensions_readings_cumulativeRealEnergy(self):
        return 1
    
    #
    # Return the number of elements in the array 'readings.cumulativeRealEnergy'
    #
    def numElements_readings_cumulativeRealEnergy():
        return 2
    
    #
    # Return the number of elements in the array 'readings.cumulativeRealEnergy'
    # for the given dimension.
    #
    def numElements_readings_cumulativeRealEnergy(self, dimension):
        array_dims = [ 2,  ]
        if dimension < 0 or dimension >= 1:
            raise IndexException
        if array_dims[dimension] == 0:
            raise IndexError
        return array_dims[dimension]
    
    #
    # Accessor methods for field: readings.averageRealPower
    #   Field type: long[]
    #   Offset (bits): 64
    #   Size of each element (bits): 32
    #

    #
    # Return whether the field 'readings.averageRealPower' is signed (False).
    #
    def isSigned_readings_averageRealPower(self):
        return False
    
    #
    # Return whether the field 'readings.averageRealPower' is an array (True).
    #
    def isArray_readings_averageRealPower(self):
        return True
    
    #
    # Return the offset (in bytes) of the field 'readings.averageRealPower'
    #
    def offset_readings_averageRealPower(self, index1):
        offset = 64
        if index1 < 0 or index1 >= 2:
            raise IndexError
        offset += 232 + index1 * 128
        return (offset / 8)
    
    #
    # Return the offset (in bits) of the field 'readings.averageRealPower'
    #
    def offsetBits_readings_averageRealPower(self, index1):
        offset = 64
        if index1 < 0 or index1 >= 2:
            raise IndexError
        offset += 232 + index1 * 128
        return offset
    
    #
    # Return the entire array 'readings.averageRealPower' as a long[]
    #
    def get_readings_averageRealPower(self):
        tmp = [None]*2
        for index0 in range (0, self.numElements_readings_averageRealPower(0)):
                tmp[index0] = self.getElement_readings_averageRealPower(index0)
        return tmp
    
    #
    # Set the contents of the array 'readings.averageRealPower' from the given long[]
    #
    def set_readings_averageRealPower(self, value):
        for index0 in range(0, len(value)):
            self.setElement_readings_averageRealPower(index0, value[index0])

    #
    # Return an element (as a long) of the array 'readings.averageRealPower'
    #
    def getElement_readings_averageRealPower(self, index1):
        return self.getUIntElement(self.offsetBits_readings_averageRealPower(index1), 32, 1)
    
    #
    # Set an element of the array 'readings.averageRealPower'
    #
    def setElement_readings_averageRealPower(self, index1, value):
        self.setUIntElement(self.offsetBits_readings_averageRealPower(index1), 32, value, 1)
    
    #
    # Return the total size, in bytes, of the array 'readings.averageRealPower'
    #
    def totalSize_readings_averageRealPower(self):
        return (256 / 8)
    
    #
    # Return the total size, in bits, of the array 'readings.averageRealPower'
    #
    def totalSizeBits_readings_averageRealPower(self):
        return 256
    
    #
    # Return the size, in bytes, of each element of the array 'readings.averageRealPower'
    #
    def elementSize_readings_averageRealPower(self):
        return (32 / 8)
    
    #
    # Return the size, in bits, of each element of the array 'readings.averageRealPower'
    #
    def elementSizeBits_readings_averageRealPower(self):
        return 32
    
    #
    # Return the number of dimensions in the array 'readings.averageRealPower'
    #
    def numDimensions_readings_averageRealPower(self):
        return 1
    
    #
    # Return the number of elements in the array 'readings.averageRealPower'
    #
    def numElements_readings_averageRealPower():
        return 2
    
    #
    # Return the number of elements in the array 'readings.averageRealPower'
    # for the given dimension.
    #
    def numElements_readings_averageRealPower(self, dimension):
        array_dims = [ 2,  ]
        if dimension < 0 or dimension >= 1:
            raise IndexException
        if array_dims[dimension] == 0:
            raise IndexError
        return array_dims[dimension]
    
    #
    # Accessor methods for field: readings.averageApparentPower
    #   Field type: long[]
    #   Offset (bits): 96
    #   Size of each element (bits): 32
    #

    #
    # Return whether the field 'readings.averageApparentPower' is signed (False).
    #
    def isSigned_readings_averageApparentPower(self):
        return False
    
    #
    # Return whether the field 'readings.averageApparentPower' is an array (True).
    #
    def isArray_readings_averageApparentPower(self):
        return True
    
    #
    # Return the offset (in bytes) of the field 'readings.averageApparentPower'
    #
    def offset_readings_averageApparentPower(self, index1):
        offset = 96
        if index1 < 0 or index1 >= 2:
            raise IndexError
        offset += 232 + index1 * 128
        return (offset / 8)
    
    #
    # Return the offset (in bits) of the field 'readings.averageApparentPower'
    #
    def offsetBits_readings_averageApparentPower(self, index1):
        offset = 96
        if index1 < 0 or index1 >= 2:
            raise IndexError
        offset += 232 + index1 * 128
        return offset
    
    #
    # Return the entire array 'readings.averageApparentPower' as a long[]
    #
    def get_readings_averageApparentPower(self):
        tmp = [None]*2
        for index0 in range (0, self.numElements_readings_averageApparentPower(0)):
                tmp[index0] = self.getElement_readings_averageApparentPower(index0)
        return tmp
    
    #
    # Set the contents of the array 'readings.averageApparentPower' from the given long[]
    #
    def set_readings_averageApparentPower(self, value):
        for index0 in range(0, len(value)):
            self.setElement_readings_averageApparentPower(index0, value[index0])

    #
    # Return an element (as a long) of the array 'readings.averageApparentPower'
    #
    def getElement_readings_averageApparentPower(self, index1):
        return self.getUIntElement(self.offsetBits_readings_averageApparentPower(index1), 32, 1)
    
    #
    # Set an element of the array 'readings.averageApparentPower'
    #
    def setElement_readings_averageApparentPower(self, index1, value):
        self.setUIntElement(self.offsetBits_readings_averageApparentPower(index1), 32, value, 1)
    
    #
    # Return the total size, in bytes, of the array 'readings.averageApparentPower'
    #
    def totalSize_readings_averageApparentPower(self):
        return (256 / 8)
    
    #
    # Return the total size, in bits, of the array 'readings.averageApparentPower'
    #
    def totalSizeBits_readings_averageApparentPower(self):
        return 256
    
    #
    # Return the size, in bytes, of each element of the array 'readings.averageApparentPower'
    #
    def elementSize_readings_averageApparentPower(self):
        return (32 / 8)
    
    #
    # Return the size, in bits, of each element of the array 'readings.averageApparentPower'
    #
    def elementSizeBits_readings_averageApparentPower(self):
        return 32
    
    #
    # Return the number of dimensions in the array 'readings.averageApparentPower'
    #
    def numDimensions_readings_averageApparentPower(self):
        return 1
    
    #
    # Return the number of elements in the array 'readings.averageApparentPower'
    #
    def numElements_readings_averageApparentPower():
        return 2
    
    #
    # Return the number of elements in the array 'readings.averageApparentPower'
    # for the given dimension.
    #
    def numElements_readings_averageApparentPower(self, dimension):
        array_dims = [ 2,  ]
        if dimension < 0 or dimension >= 1:
            raise IndexException
        if array_dims[dimension] == 0:
            raise IndexError
        return array_dims[dimension]
    



"""
Copyright (c) 2011, 2012, Regents of the University of California
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions 
are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
"""
"""
@author Stephen Dawson-Haggerty <stevedh@eecs.berkeley.edu>
"""
"""Resources for mapping sMAP objects into an HTTP server

When run as the main module, runs a sample server on port 8080
"""

import sys, os
from twisted.web import resource, server
from twisted.web.resource import NoResource
from twisted.internet import reactor, task, defer
from twisted.python import log
from twisted.python.logfile import LogFile

import uuid
import json

import util
import core
import loader
import schema
import smapconf
import actuate

from interface import *

def setResponseCode(request, exception, default):
    if hasattr(exception, 'http_code') and \
            exception.http_code != None:
        request.setResponseCode(exception.http_code)
    else:
        request.setResponseCode(default)

class InstanceResource(resource.Resource):
    """Resource which maps HTTP requests to requests on the sMAP
    instance.
    """
    def __init__(self, inst):
        self.inst = inst
        resource.Resource.__init__(self)

    isLeaf = True
    def render_GET(self, request):
        request.setHeader('Content-type', 'application/json')
        # assemble the results
        try:
            obj = self.inst.lookup(util.join_path(request.postpath))
        except Exception, e:
            setResponseCode(request, exception, 404)
            request.finish()
            return
            
        d = defer.maybeDeferred(core.SmapInstance.render_lookup, request, obj)
        d.addCallback(lambda x: self.send_reply(request, x))
        d.addErrback(lambda x: self.send_error(request, x))
        return server.NOT_DONE_YET

    def render_PUT(self, request):
        request.setHeader('Content-type', 'application/json')
        # you can only PUT actuators
        obj = self.inst.lookup(util.join_path(request.postpath),
                               pred=IActuator.providedBy)
        d = defer.maybeDeferred(core.SmapInstance.render_lookup, request, obj)
        d.addCallback(lambda x: self.send_reply(request, x))
        d.addErrback(lambda x: self.send_error(request, x))
        return server.NOT_DONE_YET


    def send_reply(self, request, obj):
        # the result out using AsyncJSON
        if obj != None:
            d = util.AsyncJSON(obj).startProducing(request)
            d.addBoth(lambda _: request.finish())
        else:
            self.send_error(request, None)

    def send_error(self, request, err):
        if err:
            setResponseCode(request, err.value, 500)
        else:
            request.setResponseCode(500)
        request.write(str(err.value))
        request.finish()

def read_report(self, request, duplicate_error=True):
    """Read a Reporting object sent by the client.  Will validate the
    object and remove extra fields which are not specified in the
    schema.
    """
    obj = schema.filter_fields('Reporting', json.load(request.content))
    if not schema.validate("Reporting", obj):
        raise core.SmapSchemaException("Invalid Reporting object (does not validate)", 400)
    if duplicate_error and self.reports.get_report(obj['uuid']):
        raise core.SmapException("Report instance already exists!", 400)
    return obj

class ReportingInstanceResource(resource.Resource):
    """Resource responsible for dealing with a single reporting instance.
    """
    def __init__(self, reports, inst):
        self.inst = inst
        self.reports = reports
        resource.Resource.__init__(self)

    def render_GET(self, request):
        """The GET verb will return the representation of the
        requested report instance.
        """
        if self.inst:
            request.setHeader('Content-type', 'application/json')
            obj = schema.filter_fields('Reporting', self.inst)
            # print schema.validate('Reporting', obj)
            d = util.AsyncJSON(obj).startProducing(request)
            d.addBoth(lambda _: request.finish())
        else:
            request.setResponseCode(404)
            request.finish()
        return server.NOT_DONE_YET

    def render_PUT(self, request):
        """The PUT verb either stores the request under the requested
        URI, or modifies an existing resource.        
        """
        try:
            request.setHeader('Content-type', 'application/json')
            obj = read_report(self, request, duplicate_error=False)
            if not self.reports.update_report(obj):
                self.reports.add_report(obj)
                request.setResponseCode(201) # created
        except Exception, e:
            setResponseCode(request, e, 400)
            request.setHeader('Content-type', 'text/plain')
            request.write(str(e))
        request.finish()
        return server.NOT_DONE_YET            

    def render_DELETE(self, request):
        """The DELETE verb remove the requested object from the collection"""
        self.reports.del_report(uuid.UUID(request.prepath[-1]))
        request.finish()
        return server.NOT_DONE_YET
            

class ReportingResource(resource.Resource):
    """Resource representing the collection of reports which are installed
    """
    def __init__(self, reports):
        self.reports = reports
        resource.Resource.__init__(self)

    def getChild(self, name, request):
        if name == '':
            return self
        try:
            id = uuid.UUID(name)
            rpt = self.reports.get_report(id)
            return ReportingInstanceResource(self.reports, rpt)
        except:
            return NoResource()

    def render_GET(self, request):
        """In response to a GET, we return a list of the installed reports
        """
        request.setHeader('Content-type', 'application/json')
        obj = {'Contents' : [x['uuid'] for x in self.reports.subscribers]}
        d = util.AsyncJSON(obj).startProducing(request)
        d.addBoth(lambda _: request.finish())
        return server.NOT_DONE_YET

    def render_POST(self, request):
        """A POST can create a new report instance as a subordinate
        resource.
        """
        try:
            request.setHeader('Content-type', 'application/json')
            obj = read_report(self, request)
            self.reports.add_report(obj)
            request.setResponseCode(201)
        except Exception, e:
            request.setHeader('Content-type', 'text/plain')
            setResponseCode(request, e, 400)
            request.write(str(e))
        request.finish()
        return server.NOT_DONE_YET

class RootResource(resource.Resource):
    """Resource representing the root of the sMAP server
    """
    def __init__(self, value=None):
        resource.Resource.__init__(self)
        if value:
            self.value = value
        else:
            self.value = {'Contents' : ['data', 'reports']}

    def getChild(self, name, request):
        if name == '':
            return self
        return resource.Resource.getChild(self, name, request) 

    def render_GET(self, request):
        request.setHeader('Content-type', 'application/json')
        return json.dumps(self.value)

def getSite(inst):
    """Return a service for creating an application
    """
    root = RootResource()
    root.putChild('data', InstanceResource(inst))
    root.putChild('reports', ReportingResource(inst.reports))

    site = server.Site(root)
    return site

def run(inst, port=None, logdir=None):
    """Start the ``twisted`` event loop, with an HTTP server.

:param inst: a :py:class:`~smap.core.SmapInstance` which you want to server.
:param int port: port to run on
:rtype: none; this function does not return
    """
    if not port: port = int(smapconf.SERVER.get('port', 8085))
    if not logdir: logdir = smapconf.SERVER.get('logdir', os.getcwd())
    if not os.path.exists(logdir):
        os.makedirs(logdir)
    print "Logging to", logdir
    print "Starting server on port", port    
    # Allow 50 1MB files
    observer = log.FileLogObserver(LogFile('sMAP.log', logdir, rotateLength=1000000, maxRotatedFiles=50))
    log.startLogging(observer)
    # Start server
    inst.start()
    reactor.listenTCP(port, getSite(inst))
    reactor.run()

if __name__ == '__main__':
    if len(sys.argv) == 2:
        # create a smap instance.  each instance needs a uuid and it should 
        s = core.SmapInstance('f83c98c0-a8c3-11e0-adf5-0026bb56ec92')

        # add collection -- easy
        # 
        # arg0 : path to collection
        # arg1 : key to generate uuid with, or Collection instance
        s.add_collection("/steve")


        # easy-add -- create a timeseries automatically.  kwargs pass through
        # to the timeseries so you can change the data type, description, etc.
        #
        # the parent must exist and be a collection for this to work.
        #
        # arg0 : path to add at
        # arg1 : either a unique string (key) or a uuid instance
        # arg2 : units
        s.add_timeseries("/sensor0", "sdh", "V")

        # alternative -- add an existing timeseries
        s.add_timeseries("/sensor1", core.Timeseries(s.uuid("sdh2"), "F", buffersz=2))

        # add readings to a timeseries
        # get_timeseries will look up based on either path or uuid
        s.get_timeseries("/sensor0").add(util.now(), 12)
        s.get_timeseries("/sensor0").add(util.now(), 13)

        # you can set timeseries properties by accessing it as a dict.  The
        # changes you make must follow the smap schema and you will get a
        # SmapSchemaException if you try to write an invalid object.
        s.get_timeseries("/sensor0")['Metadata'] = \
            {'Instrument' : {
                'Manufacturer' : "Stephen Dawson-Haggerty"
                },
             'Extra' : {
                'Sucks' : 'Andrew'
                }
             }
        s.get_collection("/")["Metadata"] = {"Extra" : {"foo" : "bar"} }

        # loader.dump(s, 'default.ini')
    else:
        s = loader.load('default.ini')
        loader.dump(s, 'foo.ini')

    counter = 0
    def newreading():
        global counter
        #print '-'*50
        s.get_collection('/')['Metadata']['Location'] = {'Room' : counter}
        s.get_collection('/').dirty_children()
        for i in xrange(0, 1):
#             s.get_timeseries('/sensor0')._add(util.now(), counter)
#             s.get_timeseries('/sensor1')._add(counter)
            s._add('/sensor0', util.now(), counter)
            s._add('/sensor1', counter)
            counter += 1
        # the default flush period is one second, so we'll just rely on that

    t = task.LoopingCall(newreading)
    t.start(10)
    log.startLogging(sys.stdout)
    run(s)




import uuid
import shelve
import operator
import datetime
import copy
import itertools

import numpy as np

from twisted.internet import reactor, threads
from smap import driver, util, smapconf
from smap.core import SmapException
from smap.archiver.client import SmapClient, RepublishClient
from smap.contrib import dtutil

from dateutil.tz import gettz

# null vector with the right shape so we can index into it
null = np.array([])
null = np.reshape(null, (0, 2))

OP_N_TO_1 = 1
OP_N_TO_N = 2

class Operator:
    # data type of operator output
    data_type = ('double', float)
    required_tags = set(['uuid'])
    optional_tags = set([])

    def __init__(self, inputs, 
                 outputs=OP_N_TO_1, 
                 tags=None):
        """
        :param inputs: list of uuids the operator examines
        """
        self.inputs = inputs
        self.tags = tags
        self._has_pending = False
        self._pending = [null] * len(inputs)

        uuids = map(operator.itemgetter('uuid'), inputs)

        # auto-construct output ids if requested
        if outputs == OP_N_TO_1:
            self.outputs = [util.dict_all(inputs)]
            self.outputs[0]['uuid'] = reduce(lambda x, y: str(uuid.uuid5(y, x)), 
                                             map(uuid.UUID, sorted(uuids)), 
                                             self.name)
        elif outputs == OP_N_TO_N:
            self.outputs = copy.deepcopy(inputs)
            for i, uid in enumerate(map(lambda x: str(uuid.uuid5(x, self.name)),
                                        map(uuid.UUID, uuids))):
                self.outputs[i]['uuid'] = uid
        else:
            self.outputs = copy.deepcopy(outputs)

    def index(self, streamid):
        for i in xrange(0, len(self.inputs)):
            if self.inputs[i]['uuid'] == streamid:
                return i
        return None

    def _name(self):
        """Return a stringified name for the operator"""
        raise NotImplementedError()

    name = property(_name)

    def __call__(self, input):
        return self.process(input)

    def reset(self):
        """Reset the internal state to discard any changes made"""
        raise NotImplementedError()

    def process(self, recs):
        """Process a number of records in bulk

:param list recs: a list of (timestamp, value) tuples
:return list: a list of (timestamp, value)  """
        raise NotImplementedError()

    def _push(self, stream, data, stream_idx=None):
        """Insert data for a single stream"""
        if not stream_idx: stream_idx = self.index(stream) 
        self._pending[stream_idx] = data
        self._has_pending = True

    def _process(self):
        """Deliver all waiting data."""
        if self._has_pending: 
            rv = self.process(self._pending)
            self._pending = [null] * len(self.inputs)
            self._has_pending = False
            return rv
        else:
            return [null] * len(self.inputs)


class OperatorDriver(driver.SmapDriver):
    """Base class for code which wants to process single streams.

    To do this you should:

     a) implement an :py:class:`Operator`, which contains your
      specific logic.  You should at least override name() to provide
      a human-readable name for your operator, and and bulk() which
      processes chunks of data..
     b) subclass OperatorDriver, implementing a "setup" method which
      creates operators and adds them using :py:method:`add_operator`.
      Make sure you call OperatorDriver.setup.

    If you do this, you'll be able to use your operator both in
    real-time mode (via a twistd smap source) and to run on historical
    data, using `smap-load` to load source data and pipe it through
    operators.
    """
    load_chunk_size = datetime.timedelta(hours=4)

    def add_operator(self, path, op):
        """Add an operator to the driver
        """
        if len(op.outputs) != 1:
            raise SmapException("Can only add operators with a single output!")
        # print op.outputs
        opid = op.outputs[0]['uuid']
        unit = op.outputs[0]['Properties/UnitofMeasure']
        # print "Added", opid
        if not isinstance(opid, uuid.UUID):
            opid = uuid.UUID(opid)

        self.add_timeseries(path, opid, unit, 
                            data_type=op.data_type[0],
                            milliseconds=False)
        self.set_metadata(path, {
                'Extra/SourceStream' : \
                    ','.join(map(operator.itemgetter('uuid'), op.inputs)),
                'Extra/Operator' : str(op.name)
                })
        self.set_metadata(path, op.outputs[0])

        for source in op.inputs:
            source = source['uuid']
            if not source in self.operators:
                self.operators[source] = {}
            if not opid in self.operators[source]:
                self.operators[source][opid] = (path, op)
        self.oplist.append((path, op))

    def reset(self):
        """Reset all operators"""
        for oplist in self.operators.itervalues():
            for path, op in oplist.itervalues():
                op.reset()

    def _data(self, data, process=True):
        """Process incoming data by pushing it through the operators
        
        process: don't actually process the operators, just add the
           pending data.
        """
        print "_data", len(data)
        for v in data.itervalues():
            if not 'uuid' in v: 
                continue
            source_id = str(v['uuid'])
            if not source_id in self.operators: 
                continue

            data = v['Readings']
            if not isinstance(data, np.ndarray):
                data = np.array(data)
            if len(data) == 0:
                data = np.reshape(data, (0, 2))
            data[:,0] /= 1000

            # push all data through the appropriate operators
            for addpath, op in self.operators[source_id].itervalues():
                op._push(source_id, data)

        if not process: return

        for addpath, op in self.oplist:
            new = op._process()
            for newv in new[0]:
                ts, v = int(newv[0]), op.data_type[1](newv[1])
                # print "adding", addpath, newv
                self._add(addpath, ts, v)

    def setup(self, opts, restrict=None, shelveoperators=False, cache=True, raw=False):
        self.source_url = opts.get('SourceUrl', 'http://ar2.openbms.org:8079')
        if not raw:
            self.restrict = '(' + restrict + ') and not has Metadata/Extra/Operator'
        else:
            self.restrict = restrict

        if shelveoperators:
            self.operators = shelve.open(opts.get('OperatorCache', '.operators'),
                                         protocol=2, writeback=True)
            # sync the operator state periodically and at exit
            util.periodicCallInThread(self.operators.sync).start(60)
            reactor.addSystemEventTrigger('after', 'shutdown', 
                                          self.operators.close)
        else:
            self.operators = {}
            self.oplist = []
        self.arclient = SmapClient(self.source_url)
        self.cache = cache

        # create timeseries from cached operator state
        for sid, oplist in self.operators.iteritems():
            for path, op in oplist.itervalues():
                self.add_operator(path, op)

    def start(self):
        """Start receiving real-time data when used in daemon mode"""
        # set up clients to provide the data
        self.client = RepublishClient(self.source_url, self._data, 
                                      restrict=self.restrict)
        self.client.connect()

        # can have multiple sources
        # RepublishClient('http://local.cs.berkeley.edu:8079', self.data).connect()

    def load(self, start_dt, end_dt, cache=True):
        """Load a range of time by pulling it from the database and
        pushing it through the operators"""
        print "starting load..."
        self.load_uids = self.operators.keys()
        self.start_dt, self.end_dt = start_dt, end_dt
        self.cache = cache
        return self.load_time_chunk(self)

    def load_time_chunk(self, *args):
        if self.start_dt >= self.end_dt:
            return None

        # pick a new window
        start = self.start_dt
        end = self.start_dt + self.load_chunk_size
        if end > self.end_dt: end = self.end_dt

        start, end  = dtutil.dt2ts(start), \
            dtutil.dt2ts(end)

        print "loading", self.start_dt, '-', self.end_dt
        self.start_dt = self.start_dt + self.load_chunk_size
        #print start, end
        #print self.load_uids
        d = threads.deferToThread(self.arclient.data_uuid, 
                                  self.load_uids,
                                  start, end, 
                                  self.cache)
        d.addCallback(self.load_data)
        d.addCallback(lambda _: self._flush())
        d.addCallback(self.load_time_chunk)
        def err(e):
            print e
        d.addErrback(err)
        return d

    def load_data(self, data):
        dobj = dict((('/%s' % str(uid), {
                    'uuid' : str(uid), 
                    'Readings' : dv}) for uid, dv in zip(self.load_uids, data)))
        self._data(dobj)


class GroupedOperatorDriver(OperatorDriver):
    """Driver which selects streams using a `Restrict` tag selector,
    and groups them according to a `Group` tagname.` It runs one
    operator per group, and send data from each stream to the proper
    operator.

    Make self.operator_class a staticmethod which instantiates a new
    operator.
    """
    operator_class = None

    def setup(self, opts):
        self.restrict = opts.get("Restrict")
        self.group = opts.get("Group")
        OperatorDriver.setup(self, opts, self.restrict, shelveoperators=False)

        # look up the streams, units, and group tags.
        client = SmapClient()
        streams = client.tags(self.restrict, '*')
                              # 'uuid, Properties/UnitofMeasure, Metadata/SourceName, %s' % 
                              # self.group)
        #print streams
        groupitems = {}

        # find the groups
        for s in streams:
            if not s[self.group] in groupitems:
                groupitems[s[self.group]] = []
            groupitems[s[self.group]].append(s)

        # instantiate one operator per group with the appropriate inputs
        for group, tags in groupitems.iteritems():
            inputs = map(operator.itemgetter('uuid'), tags)
            op = self.operator_class(tags)
            path = '/' + util.str_path(group)
            self.add_operator(path, op)


##############################################################################
##
## Operators for the framework
##
##############################################################################

def _extend(a1, a2):
    """Extend data vector a1 with vector a2"""
    assert(len(a1) == len(a2))
    rv = [None] * len(a1)
    for i in xrange(0, len(a2)):
        if len(a2[i]):
            rv[i] = np.vstack((a1[i], a2[i]))
        else:
            rv[i] = a1[i]
    return rv

def join(inputs, last=None):
    """Join together streams based on timestamps, throwing out places
    where they do not overlap"""
    times = reduce(lambda x, y: np.intersect1d(x, y[:,0]), inputs)
    vals = map(lambda x: x[np.nonzero(np.in1d(x[:,0], times)), :][0]
               if len(x) else null, inputs)
    return vals 

class ParallelSimpleOperator(Operator):
    """Parent class for operators which can be applied separately to
    each stream.  Create a classmethod called `base_operator` which
    performs the appropriate operation when called on a single stream

    Any keyword args will be passed to this classmethod on invocation;
    it needs to have a special form: it should take as an argument a
    data vector, and return a (result, kwarg) tuple.  The tuple will
    be passed to the operator on the next invocation.
    """
    def __init__(self, inputs, **initargs):
        Operator.__init__(self, inputs, outputs=OP_N_TO_N)
        self.op = parallelize(self.base_operator, 
                              len(inputs),
                              **initargs)

    def process(self, input):
        return self.op(input)


def parallelize(operator, n, **initargs):
    """Parallelize operators which perform identically on n input
    streams.
    """
    state = [initargs] * n
    def _parallelized(inputs, *kwags):
        rv = [None] * n
        for i in xrange(0, n):
            opdata = operator(inputs[i], **state[i])
            if isinstance(opdata, np.ndarray) or \
                    isinstance(opdata, list):
                rv[i] = opdata
            else:
                rv[i], state[i] = opdata
        return rv
    return _parallelized

class CompositionOperator(Operator):
    """Set an oplist property of a class instance of this, or a
    subclass The oplist should be a list of operator constructors;
    when passed in data, this class will chain the operators together
    and become an operator of their compsition.
    """
    def __init__(self, inputs):
        self.ops = []
        _inputs = inputs
        for opclass in self.oplist:
            op = opclass(_inputs)
            self.ops.append(op)
            _inputs = op.outputs
        self.required_tags = set.union(*map(lambda x: x.required_tags, self.ops))
        self.optional_tags = set.union(*map(lambda x: x.optional_tags, self.ops))

        Operator.__init__(self, inputs, _inputs)

    def process(self, data):
        return reduce(lambda x, y: y(x), self.ops, data)

class StandardizeUnitsOperator(Operator):
    operator_name = 'units'
    operator_constructors = [()]
    units = {
        'Watts' : ('kW', 0.001),
        }
    name = 'standardize units'
    required_tags = set(['uuid', 'Properties/UnitofMeasure'])

    def __init__(self, inputs):
        self.factors = [1.0] * len(inputs)
        outputs = copy.deepcopy(inputs)
        for i in xrange(0, len(inputs)):
            if 'Properties/UnitofMeasure' in inputs[i] and \
                    inputs[i]['Properties/UnitofMeasure'] in self.units:
                self.factors[i] = self.units[inputs[i]['Properties/UnitofMeasure']][1]
                outputs[i]['Properties/UnitofMeasure'] = \
                    self.units[inputs[i]['Properties/UnitofMeasure']][0]
        Operator.__init__(self, inputs, outputs)

    def process(self, data):
        return map(lambda (i, x): np.dstack((x[:, 0], x[:,1] * self.factors[i]))[0],
                   enumerate(data))

class GroupbyTimeOperator(Operator):
    """Time grouping operator.  Divide time into windows, and call
    group_operator on all the data in each window; return the result.
    The group_operator must return a single reading on each output
    stream.

    chunk_length: the size of the windows time is chunked into
    chunk_delay: when we call the group operator.
    """
    name = "latest buffer operator"
    def __init__(self, inputs, 
                 group_operator,
                 chunk_length=10,
                 chunk_delay=1):
        self.bucket_op = group_operator(inputs)
        Operator.__init__(self, inputs, outputs=self.bucket_op.outputs)
        self.pending = [null] * len(self.outputs)
        self.chunk_length = chunk_length
        self.chunk_delay = chunk_delay

    def process(self, input):
        # store the new data
        # print "processing..."
        self.pending = _extend(self.pending, input)

        # apply the grouping operator to each window
        startts = min(map(lambda x: np.min(x[:, 0]) if len(x) else np.inf,
                          self.pending))
        endts = max(map(lambda x: np.max(x[:, 0]) if len(x) else 0, 
                        self.pending))
        rv = [null] * len(self.outputs)

        if startts == np.inf or endts == 0:
            return rv

        startts = int(startts - (startts % self.chunk_length))
        endts = int((endts - (endts % self.chunk_length)) - \
                        (self.chunk_length * self.chunk_delay))
 
        # iterate over the groups
        for time in xrange(startts, endts, self.chunk_length):
            # print "group starting", time
            data = map(lambda x: x[np.where((x[:,0] >= time) & 
                                            (x[:,0] < time + self.chunk_length))] 
                       if len(x) else np.array([[time, np.nan]]),
                       self.pending)
            data = [x if len(x) else np.array([[time, np.nan]]) for x in data]
            # apply
            opresult = self.bucket_op(data)
            if max(map(len, opresult)) > 1:
                raise SmapException("Error! Grouping operators can not produce "
                                    "more than one result per group!")
            rv = _extend(rv, opresult)

        # filter out the data we operated on
        self.pending = map(lambda x: x[np.nonzero(x[:, 0] >= endts)]
                           if len(x) else null, 
                           self.pending)
        return rv

class GroupByTagOperator(Operator):
    def __init__(self, inputs, group_operator, group_tag):
        groups = [None]
        self.group_idx = [[]]
        group_inputs = [[]]
        # compute 
        for i, s in enumerate(inputs):
            if not s.get(group_tag, None) in groups:
                groups.append(s[group_tag])
                self.group_idx.append(list())
                group_inputs.append(list())

            g_idx = groups.index(s.get(group_tag, None))
            group_inputs[g_idx].append(s)
            self.group_idx[g_idx].append(i)


        self.operators = [group_operator(x) if len(x) else PrintOperator([])
                          for x in group_inputs]

        Operator.__init__(self, inputs, 
                          outputs=util.flatten(map(operator.attrgetter('outputs'), 
                                                   self.operators)))

    def process(self, data):
        rv = [[] for x in xrange(0, len(self.operators))]
        for i, op in enumerate(self.operators):
            input_data = [data[j] for j in self.group_idx[i]]
            rv[i] = self.operators[i](input_data)
        return util.flatten(rv)

class BufferedJoinOperator(Operator):
    """This operator takes an arbitrary number of streams as input,
    and combines them by joining on timestamp.  It keeps a
    variable-size join buffer in case readings come at different
    rates.
    """
    name = "buffered join"
    maxbufsz = 100
    joinmethod = staticmethod(join)

    def __init__(self, inputs):
        Operator.__init__(self, inputs, outputs=OP_N_TO_N)
        self.pending = [null] * len(inputs)
        self.last = -1

    def process(self, input):
        # add new data to the pending map
        self.pending = map(lambda (x, y): np.vstack((x,y)),
                           zip(self.pending, input))
        # find what data we can deliver
        output = self.joinmethod(self.pending, last=self.last)
        # and filter out data we no longer need
        if len(output[0]):
            self.last = np.max(output[0][:, 0])
            self.pending = map(lambda x: x[np.nonzero(x[:, 0] > self.last)],
                               self.pending)
        # truncate buffer
        self.pending = map(lambda x: x[-self.maxbufsz:], self.pending)
        return output


class _MissingDataOperator(Operator):
    """For row-wise aligned input data, only yield rows where more
    than ndatathresh percent of the streams have data (that is, are
    not nan).

    inputs: equal-length vectors with missing data represented by nan
    outputs: the same data, but only where sufficient streams have data
    """
    name = 'missing filter'
    def __init__(self, inputs, ndatathresh=0.6):
        Operator.__init__(self, inputs, OP_N_TO_N)
        self.ndatathresh = ndatathresh

    def process(self, inputs):
        # the sum of anything and a nan is nan
        times = np.vstack(map(lambda x: x[:, 1], inputs))
        nancnt = np.sum(np.isnan(times), axis=0)
        takerows = np.where(len(inputs) - nancnt >= len(inputs) * self.ndatathresh)
        if len(takerows[0]):
            return map(lambda x: x[takerows], inputs)
        else:
            return [null] * len(inputs)


def _subsample(vec, last=-1, bucketsz=5):
    if len(vec) == 0:
        return null, {'last' : last,
                      'bucketsz' : bucketsz}
    
    # ignore data before "last"
    vec[:, 0] -= np.mod(vec[:,0], bucketsz)
    times = vec[:,0]
    sp = np.where(times > last)
    if len(sp[0]) == 0: 
        return null, {'last': last,
                      'bucketsz': bucketsz}
    else: sp = sp[0][0]

    # add a dummy "last" ts
    times = np.hstack(([last], times[sp:]))
    # and bucket the times
    # we want the first point in each bucket
    takes = np.nonzero(times[1:] - times[:-1])
    rv = vec[takes[0] + sp]
    rv[:,0] = times[takes[0] + 1]
    return rv, {'last': np.max(rv[:,0]), 
                'bucketsz' : bucketsz}

class SubsampleOperator(ParallelSimpleOperator):
    """Subsample N streams in parallel by taking the first reading
    that comes in within each bucket.
    """
    # the operator we'll parallelize across all input streams
    base_operator = staticmethod(_subsample)
    def __init__(self, inputs, windowsz):
        self.name = 'subsample-%i' % windowsz
        ParallelSimpleOperator.__init__(self, inputs, 
                                        bucketsz=windowsz)

def _snaptimes(vec, bucketsz=300):
    vec[:,0] -= np.mod(vec[:,0], bucketsz)
    return vec

class SnapTimes(ParallelSimpleOperator):
    base_operator = staticmethod(_snaptimes)
    def __init__(self, inputs, windowsz):
        self.name = 'snaptimes-%i' % windowsz
        ParallelSimpleOperator.__init__(self, inputs,
                                        bucketsz=windowsz)


class PrintOperator(Operator):
    """N-N operator which prints all the input data
    """
    name = "print"
    def __init__(self, inputs):
        # don't change uuids
        Operator.__init__(self, inputs, inputs)

    def process(self, inputs):
        print inputs
        return inputs

class StripMetadata(Operator):
    name = "strip_metadata"
    operator_name = 'strip_metadata'
    operator_constructors = [()]

    def __init__(self, inputs):
        outputs = [{} for x in xrange(0, len(inputs))]
        for i, stream in enumerate(inputs):
            for k, v in stream.iteritems():
                if not k.startswith('Metadata/'):
                    outputs[i][k] = v
        Operator.__init__(self, inputs, outputs)

    def process(self, inputs):
        return inputs

class DatetimeOperator(ParallelSimpleOperator):
    required_tags = set(['uuid', 'Properties/Timezone'])
    name = 'datetime'

    def __init__(self, inputs):
        tz = set(map(operator.itemgetter('Properties/Timezone'), inputs))
        if len(tz) != 1:
            raise SmapException("Datetime operator only supports a single tz")
        self.tz = gettz(list(tz)[0])
        self.base_operator = lambda vec: self._base_operator(vec)
        ParallelSimpleOperator.__init__(self, inputs)

    def _base_operator(self, vec):
        return zip(map(lambda x: dtutil.ts2dt(x).astimezone(self.tz), 
                       map(int, vec[:,0].astype(np.int))), vec[:, 1])


if __name__ == '__main__':
    from smap.drivers.sumr import SubtractOperator
    c = SmapClient()
    q = 'Metadata/Extra/Type = "room temperature" or Metadata/Extra/Type = "room setpoint"'
    inputs = c.tags(q)
    op = GroupByTagOperator(inputs, lambda x: SubtractOperator(x, 'Metadata/Extra/Type', 300000), 'Metadata/Extra/Vav')

    data = c.latest(q, streamlimit=1000, limit = 500)
    data = [np.array(x['Readings']) for x in data]
    print op(data)



"""
Copyright (c) 2011, 2012, Regents of the University of California
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions 
are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
"""

from distutils.core import setup, Extension

# build modbus extension module
modbus_module = Extension('smap.iface.modbus._TCPModbusClient',
                          sources=map(lambda f: "smap/iface/modbus/" + f,
                                      ["TCPModbusClient_wrap.c", "TCPModbusClient.c",
                                       "utility.c", "crc16.c", "DieWithError.c",
                                       "HandleModbusTCPClient.c"]))

setup(name="Smap",
      version="2.0.266",
      description="sMAP standard library and drivers",
      author="Stephen Dawson-Haggerty",
      author_email="stevedh@eecs.berkeley.edu",
      url="http://cs.berkeley.edu/~stevedh/smap2/",
      license="BSD",
      packages=[
        # core sMAP libs and drivers
        "smap", 
        "smap.drivers", 
        "smap.contrib",

        # smap archiver components
        "smap.archiver",

        "twisted", "twisted.plugins",

        # interfaces for talking to different backends
        "smap.iface", "smap.iface.http", "smap.iface.modbus",
        "smap.iface.modbustcp",

        # hack to support ipv6 sockets -- needed for acme, at least
        "tx", "tx.ipv6", "tx.ipv6.application", "tx.ipv6.internet",

        # various extra divers and dependencies -- might want to leave this out on mainline
        "smap.drivers.obvius",
        # packages for the acme driver -- don't install this in trunk/
        "smap.drivers.acmex2", "tinyos", "tinyos.message",

        "smap.drivers.labjack", "smap.drivers.labjack.labjackpython",
        ],
      requires=["avro", "twisted", "ordereddict", "ply", "psycopg2"],
      # package_dir={"smap" : "smap"},
      package_data={"smap" : ['schema/*.av', 'archiver/sql/*.psql'], 
                    'conf': ['*.ini']},
      ext_modules=[modbus_module],
      scripts=['bin/jprint', 'bin/uuid', 'bin/smap-query', 
               'bin/smap-run-driver', 'bin/smap-load',
               'bin/smap-reporting', 'bin/smap-monitize'])



"""
Copyright (c) 2011, 2012, Regents of the University of California
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions 
are met:

 - Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
 - Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the
   distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
"""
"""
@author Stephen Dawson-Haggerty <stevedh@eecs.berkeley.edu>
"""
"""Database of sMAP mappings for sensors coming through the obvius
AcquiSuite tool.
"""

import sys
import re

from smap import util
MAYBEFLOATPAT = r'^(-?\d+(\.\d+)?)'

"""Format for the entries under 'sensors' and 'meters' is a 5-tuple consisting of:
  (obvius name,
   regular expression to parse the reading value with,
   sMAP sense point name
   sMAP channel name
   sMAP formatting object)

MAYBEFLOATPAT is a regular expression which will match either an
integer or a floating point number; a lot of things seemed to be
printed this way.
"""
DB = [
    {
        "obviusname" : "Power Measurement ION 6200",
        "sensors" : [
            ('I a (Amps)', MAYBEFLOATPAT, 'A', 'current', 'A'),
            ('I b (Amps)', MAYBEFLOATPAT, 'B', 'current', 'A'),
            ('I c (Amps)', MAYBEFLOATPAT, 'C', 'current', 'A'),
            ('I demand', MAYBEFLOATPAT, 'ABC', 'current_demand', 'A'),

            ('I a demand', MAYBEFLOATPAT, 'A', 'current_demand', 'A'),
            ('I b demand', MAYBEFLOATPAT, 'B', 'current_demand', 'A'),
            ('I c demand', MAYBEFLOATPAT, 'C', 'current_demand', 'A'),

            ('I1 THD', MAYBEFLOATPAT, 'A', 'thd', 'pct'),
            ('I2 THD', MAYBEFLOATPAT, 'B', 'thd', 'pct'),
            ('I3 THD', MAYBEFLOATPAT, 'C', 'thd', 'pct'),

            ('Frequency', MAYBEFLOATPAT, 'ABC', 'line_frequency', 'Hz'),

            ('Vll AB', r'^(\d+)', 'AB', 'volts', 'V'),
            ('Vll BC', r'^(\d+)', 'BC', 'volts', 'V'),
            ('Vll CA', r'^(\d+)', 'AC', 'volts', 'V'),

            ('kW total', r'^(\d+)', 'ABC', 'real_power', 'kW'),
            ('kVA total', r'^(\d+)', 'ABC', 'apparent_power', 'kVA'),
            ('kVAR total', r'^(\d+)', 'ABC', 'reactive_power', 'kVAR'),

            ('kW demand', r'^(\d+)', 'ABC', 'real_power_demand', 'kW'),
            ('kVA demand', r'^(\d+)', 'ABC', 'apparent_power_demand', 'kVA'),
            ('kVAR demand', r'^(\d+)', 'ABC', 'reactive_power_demand', 'kVAR'),

            ('PF sign total', r'^(-?\d+\.\d+)', 'ABC', 'pf', 'PF')
            ],

        "meters" : [
            ('kWh del', r'^(\d+)', 'ABC', 'true_energy', 'kWh'),
            ('kWh rec', r'^(\d+)', 'ABC', 'true_energy_received', 'kWh'),

            ('kVARh del', r'^(\d+)', 'ABC', 'reactive_energy', 'kVARh'),
            ('kVARh rec', r'^(\d+)', 'ABC', 'reactive_energy_received', 'kVARh'),

            ('kVAh del+rec', r'^(\d+)', 'ABC', 'apparent_energy_net', 'kVAh'),

            ('kWh a del', r'^(\d+)', 'A', 'true_energy', 'kWh'),
            ('kWh a rec', r'^(\d+)', 'A', 'true_energy_received', 'kWh'),
            ('kWh b del', r'^(\d+)', 'B', 'true_energy', 'kWh'),
            ('kWh b rec', r'^(\d+)', 'B', 'true_energy_received', 'kWh'),
            ('kWh c del', r'^(\d+)', 'C', 'true_energy', 'kWh'),
            ('kWh c rec', r'^(\d+)', 'C', 'true_energy_received', 'kWh'),
            ]
        },
    {
        'obviusname' : 'Shark 100',
        'sensors' : [
            ('Volts A-N (Volts)', MAYBEFLOATPAT, 'A', 'volts', 'V'),
            ('Volts B-N (Volts)', MAYBEFLOATPAT, 'B', 'volts', 'V'),
            ('Volts C-N (Volts)', MAYBEFLOATPAT, 'C', 'volts', 'V'),

            ('Volts A-B (Volts)', MAYBEFLOATPAT, 'AB', 'volts', 'V'),
            ('Volts B-C (Volts)', MAYBEFLOATPAT, 'BC', 'volts', 'V'),
            ('Volts C-A (Volts)', MAYBEFLOATPAT, 'AC', 'volts', 'V'),

            ('Amps A (Amps)', MAYBEFLOATPAT, 'A', 'current', 'A'),
            ('Amps B (Amps)', MAYBEFLOATPAT, 'B', 'current', 'A'),
            ('Amps C (Amps)', MAYBEFLOATPAT, 'C', 'current', 'A'),

            ('Watts, 3-Ph total', r'^(\d+)', 'ABC', 'real_power', 'kW'),
            ('VAs, 3-Ph total', r'^(\d+)', 'ABC', 'apparent_power', 'kVA'),
            ('VARs, 3-Ph total', r'^(\d+)', 'ABC', 'reactive_power', 'kVAR'),

            ('Power Factor, 3-Ph total', r'^(-?\d+\.\d+)', 'ABC', 'pf', 'PF'),

            ('Frequency (Hz)', MAYBEFLOATPAT, 'ABC', 'line_frequency', 'Hz'),

            ('Angle, Phase A Current', MAYBEFLOATPAT, 'A', 'current_phase_angle', 'deg'),
            ('Angle, Phase B Current', MAYBEFLOATPAT, 'B', 'current_phase_angle', 'deg'),
            ('Angle, Phase C Current', MAYBEFLOATPAT, 'C', 'current_phase_angle', 'deg'),

            ('Angle, Volts A-B', MAYBEFLOATPAT, 'AB', 'volts_phase_angle', 'deg'),
            ('Angle, Volts B-C', MAYBEFLOATPAT, 'BC', 'volts_phase_angle', 'deg'),
            ('Angle, Volts C-A', MAYBEFLOATPAT, 'AC', 'volts_phase_angle', 'deg'),

            ],
        'meters' : [
            ('W-hours, Received', r'^(\d+)', 'ABC', 'true_energy_received', 'kWh'),
            ('W-hours, Delivered', r'^(\d+)', 'ABC', 'true_energy_delivered', 'kWh'),

            ('VAR-hours, Positive', r'^(\d+)', 'ABC', 'reactive_energy_positive', 'kVARh+'),
            ('VAR-hours, Negative', r'^(\d+)', 'ABC', 'reactive_energy_negative', 'kVARh-'),
            ('VAR-hours, Net', r'^(\d+)', 'ABC', 'reactive_energy_net', 'kVAR net'),
            ('VAR-hours, Total', r'^(\d+)', 'ABC', 'reactive_energy_total', 'kVARh'),

            ]
        },
    {
        'obviusname' : 'AcquiSuite 8811-1 Internal 4A4P-M2',
        'locations' : ['Soda Hall'],
        'meters' : [
            ('Steam (Lbs)', r'^(\d)+', 'steam', 'total', 'Lbs'),
            ('Steam Rate', r'^(\d)+', 'steam', 'rate', 'Lbs/hr'),
            ('Electric Main #1 (#...213) (kWh)', r'^(\d)+', 'electric_1', 'true_energy_received', 'kWh'),
            ('Electric Main #1 (#...213) Demand (kW)', r'^(\d)+', 'electric_1', 'real_power', 'kW'),
            ('Electric Main #2 (#...378) (kWh)', r'^(\d)+', 'electric_2', 'true_energy_received', 'kWh'),
            ('Electric Main #2 (#...378) Demand (kW)', r'^(\d)+', 'electric_2', 'real_power', 'kW'),
            
            ],
        'sensors' : []
        },
    {
        'obviusname' : 'Power Measurement ION 7330',
        "sensors" : [
            ('I a (Amps)', MAYBEFLOATPAT, 'A', 'current', 'A'),
            ('I b (Amps)', MAYBEFLOATPAT, 'B', 'current', 'A'),
            ('I c (Amps)', MAYBEFLOATPAT, 'C', 'current', 'A'),

            ('I1 THD', MAYBEFLOATPAT, 'A', 'thd', 'pct'),
            ('I2 THD', MAYBEFLOATPAT, 'B', 'thd', 'pct'),
            ('I3 THD', MAYBEFLOATPAT, 'C', 'thd', 'pct'),

            ('Freq (Hz)', MAYBEFLOATPAT, 'ABC', 'line_frequency', 'Hz'),

            ('Vll ab', r'^(\d+)', 'AB', 'volts', 'V'),
            ('Vll bc', r'^(\d+)', 'BC', 'volts', 'V'),
            ('Vll ca', r'^(\d+)', 'AC', 'volts', 'V'),

            ('kW tot (kW)', r'^(\d+)', 'ABC', 'real_power', 'kW'),
            ('kVA tot (kVA)', r'^(\d+)', 'ABC', 'apparent_power', 'kVA'),
            ('kVAR tot (kVAR)', r'^(\d+)', 'ABC', 'reactive_power', 'kVAR'),

            ('kW td (kW)', r'^(\d+)', 'ABC', 'real_power_demand', 'kW'),
            ('kVA td (kVA)', r'^(\d+)', 'ABC', 'apparent_power_demand', 'kVA'),
            ('kVAR td (kVAR)', r'^(\d+)', 'ABC', 'reactive_power_demand', 'kVAR'),

            ('PF sign tot', r'^(-?\d+\.\d+)', 'ABC', 'pf', 'PF')
            ],

        "meters" : [
            ('kWh del (kWh)', r'^(\d+)', 'ABC', 'true_energy', 'kWh'),
            ('kWh rec (kWh)', r'^(\d+)', 'ABC', 'true_energy_received', 'kWh'),

            ('kVARh del (kVARh)', r'^(\d+)', 'ABC', 'reactive_energy', 'kVARh'),
            ('kVARh rec (kVARh)', r'^(\d+)', 'ABC', 'reactive_energy_received', 'kVARh'),

            ('kVAh', r'^(\d+)', 'ABC', 'apparent_energy_net', 'kVAh'),
            ]
        },
    {
        'obviusname' : 'Power Measurement ION 7300',
        "sensors" : [
            ('I a (Amps)', MAYBEFLOATPAT, 'A', 'current', 'A'),
            ('I b (Amps)', MAYBEFLOATPAT, 'B', 'current', 'A'),
            ('I c (Amps)', MAYBEFLOATPAT, 'C', 'current', 'A'),

            ('I1 THD', MAYBEFLOATPAT, 'A', 'thd', 'pct'),
            ('I2 THD', MAYBEFLOATPAT, 'B', 'thd', 'pct'),
            ('I3 THD', MAYBEFLOATPAT, 'C', 'thd', 'pct'),

            ('Freq (Hz)', MAYBEFLOATPAT, 'ABC', 'line_frequency', 'Hz'),

            ('Vll ab', r'^(\d+)', 'AB', 'volts', 'V'),
            ('Vll bc', r'^(\d+)', 'BC', 'volts', 'V'),
            ('Vll ca', r'^(\d+)', 'AC', 'volts', 'V'),

            ('kW tot (kW)', r'^(\d+)', 'ABC', 'real_power', 'kW'),
            ('kVA tot (kVA)', r'^(\d+)', 'ABC', 'apparent_power', 'kVA'),
            ('kVAR tot (kVAR)', r'^(\d+)', 'ABC', 'reactive_power', 'kVAR'),

            ('kW td (kW)', r'^(\d+)', 'ABC', 'real_power_demand', 'kW'),
            ('kVA td (kVA)', r'^(\d+)', 'ABC', 'apparent_power_demand', 'kVA'),
            ('kVAR td (kVAR)', r'^(\d+)', 'ABC', 'reactive_power_demand', 'kVAR'),

            ('PF sign tot', r'^(-?\d+\.\d+)', 'ABC', 'pf', 'PF')
            ],

        "meters" : [
            ('kWh del (kWh)', r'^(\d+)', 'ABC', 'true_energy', 'kWh'),
            ('kWh rec (kWh)', r'^(\d+)', 'ABC', 'true_energy_received', 'kWh'),

            ('kVARh del (kVARh)', r'^(\d+)', 'ABC', 'reactive_energy', 'kVARh'),
            ('kVARh rec (kVARh)', r'^(\d+)', 'ABC', 'reactive_energy_received', 'kVARh'),

            ('kVAh', r'^(\d+)', 'ABC', 'apparent_energy_net', 'kVAh'),
            ]
        },
    {
        "obviusname" : "Continental Control Systems LLC, WattNode MODBUS",
        "sensors" : [
            ('Energy Sum (k)', MAYBEFLOATPAT, 'ABC', 'energy_sum', 'kWh'),
            ('Power Sum (kW)', MAYBEFLOATPAT, 'ABC', 'power_sum', 'kW'),
            ('Power A (kW)', MAYBEFLOATPAT, 'A', 'energy_sum', 'kW'),
            ('Power B (kW)', MAYBEFLOATPAT, 'B', 'energy_sum', 'kW'),
            ('Power C (kW)', MAYBEFLOATPAT, 'C', 'energy_sum', 'kW'),
            ('Voltage A (Volts)', MAYBEFLOATPAT, 'A', 'volts', 'V'),
            ('Voltage B (Volts)', MAYBEFLOATPAT, 'B', 'volts', 'V'),
            ('Voltage C (Volts)', MAYBEFLOATPAT, 'C', 'volts', 'V'),
            ('Voltage Ave LL (Volts)', MAYBEFLOATPAT, 'ABC', 'volts', 'V'),
            ('Voltage A-B (Volts)', MAYBEFLOATPAT, 'AB', 'volts', 'V'),
            ('Voltage B-C (Volts)', MAYBEFLOATPAT, 'BC', 'volts', 'V'),
            ('Voltage A-C (Volts)', MAYBEFLOATPAT, 'AC', 'volts', 'V'),
            ('Frequency (Hz)', MAYBEFLOATPAT, 'ABC', 'line_frequency', 'Hz'),
            ('Energy A Net (kWh)', MAYBEFLOATPAT, 'A', 'energy_net', 'kWh'),
            ('Energy B Net (kWh)', MAYBEFLOATPAT, 'B', 'energy_net', 'kWh'),
            ('Energy C Net (kWh)', MAYBEFLOATPAT, 'C', 'energy_net', 'kWh'),
            ('Energy Pos A (kWh)', MAYBEFLOATPAT, 'A', 'energy_pos', 'kWh'),
            ('Energy Pos B (kWh)', MAYBEFLOATPAT, 'B', 'energy_pos', 'kWh'),
            ('Energy Pos C (kWh)', MAYBEFLOATPAT, 'C', 'energy_pos', 'kWh'),
            ('Energy Neg Sum (kWh)', MAYBEFLOATPAT, 'ABC', 'energy_neg', 'kWh'),
            ('Energy Neg Sum NR (kWh)', MAYBEFLOATPAT, 'ABC', 'energy_neg_nr', 'kWh'),
            ('Energy Neg A (kWh)', MAYBEFLOATPAT, 'A', 'energy_neg', 'kWh'),
            ('Energy Neg B (kWh)', MAYBEFLOATPAT, 'B', 'energy_neg', 'kWh'),
            ('Energy Neg C (kWh)', MAYBEFLOATPAT, 'C', 'energy_neg', 'kWh'),
            ('Energy Reactive Sum (kVARh)', MAYBEFLOATPAT, 'ABC', 'reactive_energy', 'kVARh'),
            ('Energy Reactive A (kVARh)', MAYBEFLOATPAT, 'A', 'reactive_energy', 'kVARh'),
            ('Energy Reactive B (kVARh)', MAYBEFLOATPAT, 'B', 'reactive_energy', 'kVARh'),
            ('Energy Reactive C (kVARh)', MAYBEFLOATPAT, 'C', 'reactive_energy', 'kVARh'),
            ('Energy Apparent Sum (kVAh)', MAYBEFLOATPAT, 'ABC', 'apparent_energy', 'kVAh'),
            ('Energy Apparent A (kVAh)', MAYBEFLOATPAT, 'A', 'apparent_energy', 'kVAh'),
            ('Energy Apparent B (kVAh)', MAYBEFLOATPAT, 'B', 'apparent_energy', 'kVAh'),
            ('Energy Apparent C (kVAh)', MAYBEFLOATPAT, 'C', 'apparent_energy', 'kVAh'),
            ('Power Factor Ave', MAYBEFLOATPAT, 'ABC', 'pf', 'PF'),
            ('Power Factor A', MAYBEFLOATPAT, 'A', 'pf', 'PF'),
            ('Power Factor B', MAYBEFLOATPAT, 'B', 'pf', 'PF'),
            ('Power Factor C', MAYBEFLOATPAT, 'C', 'pf', 'PF'),
            ('Power Reactive Sum (kVAR)', MAYBEFLOATPAT, 'ABC', 'reactive_power', 'kVAR'),
            ('Power Reactive A (kVAR)', MAYBEFLOATPAT, 'A', 'reactive_power', 'kVAR'),
            ('Power Reactive B (kVAR)', MAYBEFLOATPAT, 'B', 'reactive_power', 'kVAR'),
            ('Power Reactive C (kVAR)', MAYBEFLOATPAT, 'C', 'reactive_power', 'kVAR'),
            ('Power Apparent Sum (kVA)', MAYBEFLOATPAT, 'ABC', 'apparent_power', 'kVA'),
            ('Power Apparent A (kVA)', MAYBEFLOATPAT, 'A', 'apparent_power', 'kVA'),
            ('Power Apparent B (kVA)', MAYBEFLOATPAT, 'B', 'apparent_power', 'kVA'),
            ('Power Apparent C (kVA)', MAYBEFLOATPAT, 'C', 'apparent_power', 'kVA'),
            ('Current A (Amps)', MAYBEFLOATPAT, 'A', 'current', 'A'),
            ('Current B (Amps)', MAYBEFLOATPAT, 'B', 'current', 'A'),
            ('Current C (Amps)', MAYBEFLOATPAT, 'C', 'current', 'A'),
#             ('Demand (kW)', MAYBEFLOATPAT, 'ABC', '', ''),
#             ('Demand Min (kW)', MAYBEFLOATPAT, 'ABC', '', ''),
#             ('Demand Max (kW)', MAYBEFLOATPAT, 'ABC', '', ''),
#             ('Demand Apparent (kVA)', MAYBEFLOATPAT, 'ABC', '', ''),
#             ('Demand A (kW)', MAYBEFLOATPAT, 'ABC', '', ''),
#             ('Demand B (kW)', MAYBEFLOATPAT, 'ABC', '', ''),
#             ('Demand C (kW)', MAYBEFLOATPAT, 'ABC', '', '') 
            ],
        "meters" : [],
        "extra" : {
            "Rate": 300
            }
        }
    ]

unit_replace = [
    ("^[kK][wW][hH]", "kWh"),
    ("^[Ll]bs\.?", "lbs"),
    ("^lb[^s ]", "lbs"),
    ("^Pounds", "lbs"),
    (" per ", "/"),
    ("minute", "min"),
    ("^[cC]ubic [fF]t", "ft3"),
    ("^CFm$", "ft3/min"),
    ("^CF$", "ft3"),
    ("^Cub feet", "ft3"),
    ("^[Gg]al", "Gal"),
    ("^[Gg]allons", "Gal"),
#    ("^[Gg]pm", "Gal/min"),
    ("^[cC][fF]", "CF"),
    ]    

def guess_conf(type, location, header):
    print >>sys.stderr, "l", location, "t", type
    if type.startswith("Obvius, A8812"):
        conf = { "sensors" : [],  "meters" : [], "extra" : {"Rate": 300} }
        if header:
            # make a guessed config based on the header
            for col in header:
                m = re.match("^(.*)\((.*)\)$", col)
                if m:
                    name, unit = m.groups(0)
                    for pat, rep in unit_replace:
                        unit = re.sub(pat, rep, unit)
                    name = name.strip()
                    if name.endswith("Min") or \
                            name.endswith("Max") or \
                            name.startswith("time"): 
                        continue
                    print "%s ... %s ... %s" % (col, name, unit)
                    conf["sensors"].append((col, MAYBEFLOATPAT, "", util.str_path(name), unit))
        return conf
#     elif type.startswith("AcquiSuite 8811-1"):
#         if not header: return True
#         print header
#     elif type.startswith("Obvius, ModHopper, R9120"):
#         if not header: return True
#         print header
    return None

TYPES = [x['obviusname'] for x in DB]
def get_map(type, location=None, header=None):
    for m in DB:
        if type.startswith(m['obviusname']) and ( \
            location == None or not 'locations' in m or location in m['locations']):
            return m
    return guess_conf(type, location, header)




#!/usr/bin/env python
# coding: utf-8

'''
This module initializes and installs the Twisted reactor and then conditionally
monkey patches it if the listenTCP6() method has not already been defined. This
may break when using newer versions of Twisted and will likely not work in the
circumstance that IPv6 support is finally added to Twisted. Read the release
notes for your version of Twisted to determine whether IPv6 is supported. Note
that Twisted will most likely implement IPv6 as an extension to listenTCP() and
friends instead of implementing additional methods. Use of this patch is
unsupported.

Also, this patch will probably break things.

It should be possible to use additional reactors with this patch. For example,
if you wish to use epoll instead of the standard select reactor, the following
should work if you have a fairly recent Linux-based platform:

from twisted.internet import epollreactor
epollreactor.install()
from twistedpatch.internet import reactor # Will now use epollreactor

kqueue should work similarly.

Address resolution will NOT work and hasn't been implemented, so you may need
to use Python's socket module where appropriate.
'''

import sys
del sys.modules['tx.ipv6.internet.reactor']
import sys # Required to work around some import-related funkiness.  
from twisted.internet import selectreactor

reactor = selectreactor.SelectReactor()

import tx.ipv6.internet
assert not sys.modules.has_key('tx.ipv6.internet.reactor'), \
       "reactor already installed"
if sys.modules.has_key("twisted.internet.reactor"):
    reactor = sys.modules["twisted.internet.reactor"]

tx.ipv6.internet.reactor = reactor
sys.modules['tx.ipv6.internet.reactor'] = reactor

import types
from tx.ipv6.internet import tcp6, udp6
from tx.ipv6.internet import posixbase6

if not hasattr(reactor, "listenTCP6"):
    reactor.listenTCP6 = types.MethodType(posixbase6.listenTCP6, reactor, reactor.__class__)

if not hasattr(reactor, "listenUDP6"):
    reactor.listenUDP6 = types.MethodType(posixbase6.listenUDP6, reactor, reactor.__class__)
